"""Tests for the type serialization system."""

from __future__ import annotations

import time
from datetime import date, datetime, time as dt_time, timedelta
from decimal import Decimal
from pathlib import Path, PurePath
from typing import Any
from uuid import UUID

import pytest

from auto_mcp.types import (
    AutoCompressRegistry,
    ByteArrayAdapter,
    BytesAdapter,
    CompressedAdapter,
    CompressedData,
    CompressionAlgorithm,
    CompressionConfig,
    ComplexAdapter,
    DateAdapter,
    DateTimeAdapter,
    DecimalAdapter,
    FrozenSetAdapter,
    FunctionAdapter,
    FunctionWrapper,
    ObjectStore,
    ObjectStoreConfig,
    PathAdapter,
    PurePathAdapter,
    SetAdapter,
    StoredObject,
    StoreStats,
    TimeAdapter,
    TimeDeltaAdapter,
    TypeAdapter,
    TypeInfo,
    TypeRegistry,
    TypeStrategy,
    TypeTransformError,
    UUIDAdapter,
    compress_bytes,
    decompress_bytes,
    get_all_adapters,
    get_default_registry,
    get_default_store,
    get_stdlib_adapters,
    is_json_compatible,
    register_all_adapters,
    register_stdlib_adapters,
    set_default_registry,
    set_default_store,
    type_adapter,
    with_compression,
    wrap_function,
)


class TestTypeStrategy:
    """Tests for TypeStrategy enum."""

    def test_strategy_values(self) -> None:
        """Test strategy enum values."""
        assert TypeStrategy.ADAPTER.value == "adapter"
        assert TypeStrategy.OBJECT_STORE.value == "object_store"
        assert TypeStrategy.PASSTHROUGH.value == "passthrough"
        assert TypeStrategy.UNSUPPORTED.value == "unsupported"


class TestTypeInfo:
    """Tests for TypeInfo dataclass."""

    def test_type_info_creation(self) -> None:
        """Test creating TypeInfo."""
        info = TypeInfo(
            type_=int,
            strategy=TypeStrategy.PASSTHROUGH,
            json_schema={"type": "integer"},
        )
        assert info.type_ is int
        assert info.strategy == TypeStrategy.PASSTHROUGH
        assert info.json_schema == {"type": "integer"}

    def test_type_name_property(self) -> None:
        """Test type_name property."""
        info = TypeInfo(type_=datetime, strategy=TypeStrategy.ADAPTER)
        assert "datetime" in info.type_name


class TestObjectStoreConfig:
    """Tests for ObjectStoreConfig dataclass."""

    def test_default_config(self) -> None:
        """Test default configuration values."""
        config = ObjectStoreConfig()
        assert config.ttl == 3600
        assert config.max_instances == 100
        assert config.singleton is False
        assert config.auto_create_tool is True
        assert config.auto_cleanup_tool is True
        assert config.factory is None

    def test_custom_config(self) -> None:
        """Test custom configuration."""
        config = ObjectStoreConfig(
            ttl=7200,
            max_instances=50,
            singleton=True,
        )
        assert config.ttl == 7200
        assert config.max_instances == 50
        assert config.singleton is True


class TestFunctionAdapter:
    """Tests for FunctionAdapter class."""

    def test_basic_adapter(self) -> None:
        """Test basic function adapter."""
        adapter = FunctionAdapter(
            target_type=datetime,
            serializer=lambda dt: dt.isoformat(),
            deserializer=lambda s: datetime.fromisoformat(s),
            schema={"type": "string", "format": "date-time"},
        )

        now = datetime.now()
        serialized = adapter.serialize(now)
        assert isinstance(serialized, str)

        deserialized = adapter.deserialize(serialized)
        assert isinstance(deserialized, datetime)

    def test_json_schema(self) -> None:
        """Test JSON schema generation."""
        adapter = FunctionAdapter(
            target_type=int,
            serializer=lambda x: x,
            deserializer=lambda x: x,
            schema={"type": "integer"},
        )
        schema = adapter.json_schema()
        assert schema == {"type": "integer"}

    def test_can_serialize(self) -> None:
        """Test can_serialize with custom function."""
        adapter = FunctionAdapter(
            target_type=str,
            serializer=lambda s: s,
            deserializer=lambda s: s,
            can_serialize_fn=lambda s: len(s) < 100,
        )

        assert adapter.can_serialize("short") is True
        assert adapter.can_serialize("x" * 200) is False


class TestTypeAdapterDecorator:
    """Tests for the @type_adapter decorator."""

    def test_decorator(self) -> None:
        """Test creating adapter with decorator."""

        @type_adapter(datetime)
        class CustomDateTimeAdapter:
            def serialize(self, obj: datetime) -> str:
                return obj.isoformat()

            def deserialize(self, data: str) -> datetime:
                return datetime.fromisoformat(data)

            def json_schema(self) -> dict[str, Any]:
                return {"type": "string"}

        adapter = CustomDateTimeAdapter()
        assert adapter.target_type is datetime


class TestIsJsonCompatible:
    """Tests for is_json_compatible function."""

    def test_primitives(self) -> None:
        """Test primitive types."""
        assert is_json_compatible(str) is True
        assert is_json_compatible(int) is True
        assert is_json_compatible(float) is True
        assert is_json_compatible(bool) is True
        assert is_json_compatible(type(None)) is True
        assert is_json_compatible(list) is True
        assert is_json_compatible(dict) is True

    def test_non_compatible(self) -> None:
        """Test non-compatible types."""
        assert is_json_compatible(datetime) is False
        assert is_json_compatible(Path) is False


# ============================================================================
# Type Registry Tests
# ============================================================================


class TestTypeRegistry:
    """Tests for TypeRegistry class."""

    def test_empty_registry(self) -> None:
        """Test empty registry initialization."""
        registry = TypeRegistry()
        assert registry.list_adapters() == []
        assert registry.list_stored_types() == []

    def test_register_adapter(self) -> None:
        """Test registering an adapter."""
        registry = TypeRegistry()
        adapter = DateTimeAdapter()
        registry.register_adapter(adapter)

        assert registry.has_adapter(datetime)
        assert registry.get_adapter(datetime) is adapter

    def test_register_function(self) -> None:
        """Test quick registration with functions."""
        registry = TypeRegistry()
        registry.register(
            Path,
            serialize=lambda p: str(p),
            deserialize=lambda s: Path(s),
            schema={"type": "string"},
        )

        assert registry.has_adapter(Path)

    def test_register_stored_type(self) -> None:
        """Test registering a stored type."""
        registry = TypeRegistry()

        class MyService:
            pass

        registry.register_stored_type(
            MyService,
            ttl=7200,
            max_instances=10,
            singleton=True,
        )

        assert registry.has_store_config(MyService)
        config = registry.get_store_config(MyService)
        assert config is not None
        assert config.ttl == 7200
        assert config.singleton is True

    def test_get_type_info_passthrough(self) -> None:
        """Test getting info for passthrough types."""
        registry = TypeRegistry()
        info = registry.get_type_info(int)

        assert info.strategy == TypeStrategy.PASSTHROUGH
        assert info.json_schema == {"type": "integer"}

    def test_get_type_info_adapter(self) -> None:
        """Test getting info for adapter types."""
        registry = TypeRegistry()
        registry.register_adapter(DateTimeAdapter())

        info = registry.get_type_info(datetime)
        assert info.strategy == TypeStrategy.ADAPTER
        assert info.adapter is not None

    def test_get_type_info_object_store(self) -> None:
        """Test getting info for stored types."""
        registry = TypeRegistry()

        class Session:
            pass

        registry.register_stored_type(Session)

        info = registry.get_type_info(Session)
        assert info.strategy == TypeStrategy.OBJECT_STORE
        assert info.store_config is not None

    def test_serialize(self) -> None:
        """Test serialization via registry."""
        registry = TypeRegistry()
        registry.register_adapter(DateTimeAdapter())

        now = datetime.now()
        serialized = registry.serialize(now)
        assert isinstance(serialized, str)

    def test_deserialize(self) -> None:
        """Test deserialization via registry."""
        registry = TypeRegistry()
        registry.register_adapter(DateTimeAdapter())

        iso_str = "2024-01-15T10:30:00"
        dt = registry.deserialize(iso_str, datetime)
        assert isinstance(dt, datetime)

    def test_get_json_schema(self) -> None:
        """Test getting JSON schema."""
        registry = TypeRegistry()
        registry.register_adapter(DateTimeAdapter())

        schema = registry.get_json_schema(datetime)
        assert "type" in schema

    def test_clear(self) -> None:
        """Test clearing the registry."""
        registry = TypeRegistry()
        registry.register_adapter(DateTimeAdapter())
        registry.clear()

        assert registry.list_adapters() == []


class TestDefaultRegistry:
    """Tests for default registry functions."""

    def test_get_default_registry(self) -> None:
        """Test getting default registry."""
        registry = get_default_registry()
        assert isinstance(registry, TypeRegistry)

    def test_set_default_registry(self) -> None:
        """Test setting default registry."""
        new_registry = TypeRegistry()
        set_default_registry(new_registry)
        assert get_default_registry() is new_registry


# ============================================================================
# Object Store Tests
# ============================================================================


class TestStoredObject:
    """Tests for StoredObject dataclass."""

    def test_stored_object_creation(self) -> None:
        """Test creating a stored object."""
        obj = StoredObject(
            obj="test",
            type_name="str",
            created_at=time.time(),
            expires_at=time.time() + 3600,
        )
        assert obj.obj == "test"
        assert obj.access_count == 0

    def test_is_expired(self) -> None:
        """Test expiration check."""
        # Not expired
        obj = StoredObject(
            obj="test",
            type_name="str",
            created_at=time.time(),
            expires_at=time.time() + 3600,
        )
        assert obj.is_expired is False

        # Expired
        obj_expired = StoredObject(
            obj="test",
            type_name="str",
            created_at=time.time() - 7200,
            expires_at=time.time() - 3600,
        )
        assert obj_expired.is_expired is True

        # Never expires
        obj_never = StoredObject(
            obj="test",
            type_name="str",
            created_at=time.time(),
            expires_at=0,
        )
        assert obj_never.is_expired is False

    def test_touch(self) -> None:
        """Test updating access stats."""
        obj = StoredObject(
            obj="test",
            type_name="str",
            created_at=time.time(),
            expires_at=0,
        )
        obj.touch()
        assert obj.access_count == 1
        assert obj.last_accessed > 0


class TestStoreStats:
    """Tests for StoreStats dataclass."""

    def test_default_stats(self) -> None:
        """Test default stats."""
        stats = StoreStats()
        assert stats.total_objects == 0
        assert stats.by_type == {}
        assert stats.expired_count == 0


class TestObjectStore:
    """Tests for ObjectStore class."""

    def test_store_and_get(self) -> None:
        """Test storing and retrieving objects."""
        store = ObjectStore()
        handle = store.store("test_value")

        assert store.exists(handle)
        assert store.get(handle) == "test_value"

    def test_store_with_ttl(self) -> None:
        """Test storing with TTL."""
        store = ObjectStore()
        handle = store.store("test", ttl=3600)

        info = store.get_info(handle)
        assert info["ttl_remaining"] is not None
        assert info["ttl_remaining"] > 0

    def test_store_custom_handle(self) -> None:
        """Test storing with custom handle."""
        store = ObjectStore()
        handle = store.store("test", handle="custom:handle")

        assert handle == "custom:handle"
        assert store.get(handle) == "test"

    def test_get_typed(self) -> None:
        """Test typed retrieval."""
        store = ObjectStore()
        handle = store.store("test")

        value = store.get_typed(handle, str)
        assert value == "test"

        with pytest.raises(TypeError):
            store.get_typed(handle, int)

    def test_release(self) -> None:
        """Test releasing objects."""
        store = ObjectStore()
        handle = store.store("test")

        assert store.release(handle) is True
        assert store.exists(handle) is False
        assert store.release(handle) is False

    def test_list_handles(self) -> None:
        """Test listing handles."""
        store = ObjectStore()
        h1 = store.store("test1")
        h2 = store.store("test2")

        handles = store.list_handles()
        assert h1 in handles
        assert h2 in handles

    def test_count(self) -> None:
        """Test counting objects."""
        store = ObjectStore()
        store.store("test1")
        store.store("test2")

        assert store.count() == 2

    def test_get_stats(self) -> None:
        """Test getting statistics."""
        store = ObjectStore()
        store.store("test1")
        store.store("test2")

        stats = store.get_stats()
        assert stats.total_objects == 2

    def test_cleanup(self) -> None:
        """Test cleanup of expired objects."""
        store = ObjectStore(auto_cleanup=False)
        handle = store.store("test", ttl=0)  # Expires immediately

        # Manually expire the object
        store._objects[handle].expires_at = time.time() - 1

        removed = store.cleanup()
        assert removed == 1

    def test_clear(self) -> None:
        """Test clearing all objects."""
        store = ObjectStore()
        store.store("test1")
        store.store("test2")

        count = store.clear()
        assert count == 2
        assert store.count() == 0

    def test_singleton_type(self) -> None:
        """Test singleton type constraint."""
        store = ObjectStore()
        config = ObjectStoreConfig(singleton=True)
        store.configure_type(str, config)

        h1 = store.store("first")
        h2 = store.store("second")

        # Should return same handle for singleton
        assert h1 == h2

    def test_max_instances(self) -> None:
        """Test max instances constraint."""
        store = ObjectStore()
        config = ObjectStoreConfig(max_instances=2)
        store.configure_type(str, config)

        store.store("first")
        store.store("second")

        with pytest.raises(ValueError, match="Maximum instances"):
            store.store("third")

    def test_key_not_found(self) -> None:
        """Test KeyError for missing handle."""
        store = ObjectStore()

        with pytest.raises(KeyError):
            store.get("nonexistent")

    def test_expired_object_access(self) -> None:
        """Test accessing expired objects."""
        store = ObjectStore(auto_cleanup=False)
        handle = store.store("test", ttl=1)

        # Manually expire
        store._objects[handle].expires_at = time.time() - 1

        with pytest.raises(ValueError, match="expired"):
            store.get(handle)


class TestDefaultStore:
    """Tests for default store functions."""

    def test_get_default_store(self) -> None:
        """Test getting default store."""
        store = get_default_store()
        assert isinstance(store, ObjectStore)

    def test_set_default_store(self) -> None:
        """Test setting default store."""
        new_store = ObjectStore()
        set_default_store(new_store)
        assert get_default_store() is new_store


# ============================================================================
# Built-in Adapter Tests
# ============================================================================


class TestDateTimeAdapter:
    """Tests for DateTimeAdapter."""

    def test_serialize(self) -> None:
        """Test serialization."""
        adapter = DateTimeAdapter()
        dt = datetime(2024, 1, 15, 10, 30, 0)
        serialized = adapter.serialize(dt)
        assert serialized == "2024-01-15T10:30:00"

    def test_deserialize(self) -> None:
        """Test deserialization."""
        adapter = DateTimeAdapter()
        dt = adapter.deserialize("2024-01-15T10:30:00")
        assert dt == datetime(2024, 1, 15, 10, 30, 0)

    def test_json_schema(self) -> None:
        """Test JSON schema."""
        adapter = DateTimeAdapter()
        schema = adapter.json_schema()
        assert schema["type"] == "string"
        assert schema["format"] == "date-time"


class TestDateAdapter:
    """Tests for DateAdapter."""

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = DateAdapter()
        d = date(2024, 1, 15)
        serialized = adapter.serialize(d)
        deserialized = adapter.deserialize(serialized)
        assert deserialized == d


class TestTimeAdapter:
    """Tests for TimeAdapter."""

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = TimeAdapter()
        t = dt_time(10, 30, 0)
        serialized = adapter.serialize(t)
        deserialized = adapter.deserialize(serialized)
        assert deserialized == t


class TestTimeDeltaAdapter:
    """Tests for TimeDeltaAdapter."""

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = TimeDeltaAdapter()
        td = timedelta(hours=2, minutes=30)
        serialized = adapter.serialize(td)
        assert serialized == td.total_seconds()
        deserialized = adapter.deserialize(serialized)
        assert deserialized == td


class TestPathAdapter:
    """Tests for PathAdapter."""

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = PathAdapter()
        p = Path("/home/user/file.txt")
        serialized = adapter.serialize(p)
        assert serialized == "/home/user/file.txt"
        deserialized = adapter.deserialize(serialized)
        assert deserialized == p


class TestPurePathAdapter:
    """Tests for PurePathAdapter."""

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = PurePathAdapter()
        p = PurePath("/home/user/file.txt")
        serialized = adapter.serialize(p)
        deserialized = adapter.deserialize(serialized)
        assert str(deserialized) == str(p)


class TestUUIDAdapter:
    """Tests for UUIDAdapter."""

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = UUIDAdapter()
        u = UUID("12345678-1234-5678-1234-567812345678")
        serialized = adapter.serialize(u)
        assert serialized == "12345678-1234-5678-1234-567812345678"
        deserialized = adapter.deserialize(serialized)
        assert deserialized == u


class TestDecimalAdapter:
    """Tests for DecimalAdapter."""

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = DecimalAdapter()
        d = Decimal("123.456")
        serialized = adapter.serialize(d)
        assert serialized == "123.456"
        deserialized = adapter.deserialize(serialized)
        assert deserialized == d


class TestBytesAdapter:
    """Tests for BytesAdapter."""

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = BytesAdapter()
        b = b"hello world"
        serialized = adapter.serialize(b)
        assert isinstance(serialized, str)
        deserialized = adapter.deserialize(serialized)
        assert deserialized == b

    def test_max_size(self) -> None:
        """Test max size constraint."""
        adapter = BytesAdapter(max_size=10)
        assert adapter.can_serialize(b"short") is True
        assert adapter.can_serialize(b"x" * 20) is False


class TestByteArrayAdapter:
    """Tests for ByteArrayAdapter."""

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = ByteArrayAdapter()
        ba = bytearray(b"hello world")
        serialized = adapter.serialize(ba)
        deserialized = adapter.deserialize(serialized)
        assert deserialized == ba


class TestSetAdapter:
    """Tests for SetAdapter."""

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = SetAdapter()
        s = {1, 2, 3}
        serialized = adapter.serialize(s)
        assert isinstance(serialized, list)
        deserialized = adapter.deserialize(serialized)
        assert deserialized == s


class TestFrozenSetAdapter:
    """Tests for FrozenSetAdapter."""

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = FrozenSetAdapter()
        fs = frozenset({1, 2, 3})
        serialized = adapter.serialize(fs)
        deserialized = adapter.deserialize(serialized)
        assert deserialized == fs


class TestComplexAdapter:
    """Tests for ComplexAdapter."""

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = ComplexAdapter()
        c = complex(3, 4)
        serialized = adapter.serialize(c)
        assert serialized == {"real": 3.0, "imag": 4.0}
        deserialized = adapter.deserialize(serialized)
        assert deserialized == c


class TestAdapterHelpers:
    """Tests for adapter helper functions."""

    def test_get_stdlib_adapters(self) -> None:
        """Test getting stdlib adapters."""
        adapters = get_stdlib_adapters()
        assert len(adapters) > 0
        assert any(isinstance(a, DateTimeAdapter) for a in adapters)

    def test_get_all_adapters(self) -> None:
        """Test getting all adapters."""
        adapters = get_all_adapters()
        assert len(adapters) >= len(get_stdlib_adapters())

    def test_register_stdlib_adapters(self) -> None:
        """Test registering stdlib adapters."""
        registry = TypeRegistry()
        register_stdlib_adapters(registry)
        assert registry.has_adapter(datetime)
        assert registry.has_adapter(Path)

    def test_register_all_adapters(self) -> None:
        """Test registering all adapters."""
        registry = TypeRegistry()
        register_all_adapters(registry)
        assert registry.has_adapter(datetime)


# ============================================================================
# Function Wrapper Tests
# ============================================================================


class TestFunctionWrapper:
    """Tests for FunctionWrapper class."""

    def test_basic_wrapping(self) -> None:
        """Test basic function wrapping."""

        def add(a: int, b: int) -> int:
            return a + b

        wrapper = FunctionWrapper(add)
        result = wrapper.call({"a": 1, "b": 2})
        assert result == 3

    def test_type_transformation(self) -> None:
        """Test type transformation with adapters."""
        registry = TypeRegistry()
        register_stdlib_adapters(registry)

        def get_date_str(d: datetime) -> str:
            return d.strftime("%Y-%m-%d")

        wrapper = FunctionWrapper(get_date_str, registry=registry)

        # Call with serialized datetime
        result = wrapper.call({"d": "2024-01-15T10:30:00"})
        assert result == "2024-01-15"

    def test_get_json_schema(self) -> None:
        """Test getting JSON schema for parameters."""

        def func(name: str, count: int) -> dict:
            return {"name": name, "count": count}

        wrapper = FunctionWrapper(func)
        schema = wrapper.get_json_schema()

        assert "properties" in schema
        assert "name" in schema["properties"]
        assert "count" in schema["properties"]
        assert "required" in schema

    def test_wrap_function_helper(self) -> None:
        """Test wrap_function convenience function."""

        def greet(name: str) -> str:
            return f"Hello, {name}!"

        wrapper = wrap_function(greet)
        result = wrapper.call({"name": "World"})
        assert result == "Hello, World!"

    def test_transform_error(self) -> None:
        """Test TypeTransformError."""
        error = TypeTransformError("Test error", param_name="test")
        assert str(error) == "Test error"
        assert error.param_name == "test"


class TestTypeTransformError:
    """Tests for TypeTransformError."""

    def test_error_creation(self) -> None:
        """Test creating transform error."""
        error = TypeTransformError(
            "Failed to transform",
            param_name="data",
            original_error=ValueError("invalid"),
        )
        assert "Failed to transform" in str(error)
        assert error.param_name == "data"
        assert isinstance(error.original_error, ValueError)


# ============================================================================
# Compression Tests
# ============================================================================


class TestCompressionAlgorithm:
    """Tests for CompressionAlgorithm enum."""

    def test_algorithm_values(self) -> None:
        """Test algorithm enum values."""
        assert CompressionAlgorithm.GZIP.value == "gzip"
        assert CompressionAlgorithm.ZLIB.value == "zlib"
        assert CompressionAlgorithm.LZ4.value == "lz4"
        assert CompressionAlgorithm.NONE.value == "none"


class TestCompressionConfig:
    """Tests for CompressionConfig dataclass."""

    def test_default_config(self) -> None:
        """Test default configuration values."""
        config = CompressionConfig()
        assert config.enabled is True
        assert config.algorithm == CompressionAlgorithm.GZIP
        assert config.threshold == 1024
        assert config.level == 6

    def test_custom_config(self) -> None:
        """Test custom configuration."""
        config = CompressionConfig(
            enabled=True,
            algorithm=CompressionAlgorithm.ZLIB,
            threshold=2048,
            level=9,
        )
        assert config.algorithm == CompressionAlgorithm.ZLIB
        assert config.threshold == 2048
        assert config.level == 9


class TestCompressedData:
    """Tests for CompressedData dataclass."""

    def test_to_dict(self) -> None:
        """Test converting to dict."""
        data = CompressedData(
            compressed=True,
            algorithm="gzip",
            original_size=1000,
            compressed_size=500,
            data="SGVsbG8=",
        )
        d = data.to_dict()
        assert d["compressed"] is True
        assert d["algorithm"] == "gzip"
        assert d["original_size"] == 1000
        assert d["compressed_size"] == 500
        assert d["compression_ratio"] == 0.5

    def test_from_dict(self) -> None:
        """Test creating from dict."""
        d = {
            "compressed": True,
            "algorithm": "gzip",
            "original_size": 1000,
            "compressed_size": 500,
            "data": "SGVsbG8=",
        }
        data = CompressedData.from_dict(d)
        assert data.compressed is True
        assert data.algorithm == "gzip"


class TestCompressionFunctions:
    """Tests for compression utility functions."""

    def test_compress_decompress_gzip(self) -> None:
        """Test gzip compression roundtrip."""
        original = b"Hello, World! " * 100
        compressed = compress_bytes(original, CompressionAlgorithm.GZIP)
        decompressed = decompress_bytes(compressed, CompressionAlgorithm.GZIP)
        assert decompressed == original
        assert len(compressed) < len(original)

    def test_compress_decompress_zlib(self) -> None:
        """Test zlib compression roundtrip."""
        original = b"Test data " * 100
        compressed = compress_bytes(original, CompressionAlgorithm.ZLIB)
        decompressed = decompress_bytes(compressed, CompressionAlgorithm.ZLIB)
        assert decompressed == original
        assert len(compressed) < len(original)

    def test_compress_none(self) -> None:
        """Test no compression."""
        original = b"Hello"
        result = compress_bytes(original, CompressionAlgorithm.NONE)
        assert result == original


class TestCompressedAdapter:
    """Tests for CompressedAdapter."""

    def test_small_data_not_compressed(self) -> None:
        """Test that small data is not compressed."""
        adapter = DateTimeAdapter()
        config = CompressionConfig(threshold=10000)  # High threshold
        compressed = CompressedAdapter(adapter, config)

        dt = datetime(2024, 1, 15, 10, 30, 0)
        result = compressed.serialize(dt)

        assert isinstance(result, dict)
        assert result["compressed"] is False

    def test_large_data_compressed(self) -> None:
        """Test that large data is compressed."""
        # Create a simple adapter for strings
        adapter = FunctionAdapter(
            target_type=str,
            serializer=lambda s: s,
            deserializer=lambda s: s,
            schema={"type": "string"},
        )
        config = CompressionConfig(threshold=100)  # Low threshold
        compressed = CompressedAdapter(adapter, config)

        # Create a large string
        large_string = "x" * 1000
        result = compressed.serialize(large_string)

        assert isinstance(result, dict)
        assert result["compressed"] is True
        assert result["compressed_size"] < result["original_size"]

    def test_roundtrip(self) -> None:
        """Test serialize/deserialize roundtrip."""
        adapter = DateTimeAdapter()
        config = CompressionConfig(threshold=1)  # Force compression
        compressed = CompressedAdapter(adapter, config)

        dt = datetime(2024, 1, 15, 10, 30, 0)
        serialized = compressed.serialize(dt)
        deserialized = compressed.deserialize(serialized)

        assert deserialized == dt

    def test_deserialize_uncompressed(self) -> None:
        """Test deserializing uncompressed data."""
        adapter = DateTimeAdapter()
        compressed = CompressedAdapter(adapter)

        # Pass through non-compressed data
        result = compressed.deserialize("2024-01-15T10:30:00")
        assert result == datetime(2024, 1, 15, 10, 30, 0)

    def test_json_schema(self) -> None:
        """Test JSON schema generation."""
        adapter = DateTimeAdapter()
        compressed = CompressedAdapter(adapter)

        schema = compressed.json_schema()
        assert "properties" in schema
        assert "compressed" in schema["properties"]
        assert "algorithm" in schema["properties"]
        assert "data" in schema["properties"]

    def test_can_serialize(self) -> None:
        """Test can_serialize delegation."""
        adapter = DateTimeAdapter()
        compressed = CompressedAdapter(adapter)

        assert compressed.can_serialize(datetime.now()) is True


class TestWithCompression:
    """Tests for with_compression helper function."""

    def test_with_compression(self) -> None:
        """Test convenience wrapper function."""
        adapter = DateTimeAdapter()
        compressed = with_compression(adapter, threshold=100)

        assert isinstance(compressed, CompressedAdapter)
        assert compressed.config.threshold == 100


class TestAutoCompressRegistry:
    """Tests for AutoCompressRegistry."""

    def test_register_large_type(self) -> None:
        """Test registering large types."""
        registry = AutoCompressRegistry()

        class LargeData:
            pass

        registry.register_large_type(LargeData)

    def test_wrap_adapter(self) -> None:
        """Test wrapping adapters."""
        registry = AutoCompressRegistry()
        adapter = DateTimeAdapter()

        # Not registered as large, should return as-is
        wrapped = registry.wrap_adapter(adapter)
        assert wrapped is adapter

    def test_wrap_if_large(self) -> None:
        """Test explicitly wrapping with compression."""
        registry = AutoCompressRegistry()
        adapter = DateTimeAdapter()

        wrapped = registry.wrap_if_large(adapter, threshold=500)
        assert isinstance(wrapped, CompressedAdapter)
        assert wrapped.config.threshold == 500

    def test_wrap_adapter_for_registered_large_type(self) -> None:
        """Test wrapping adapters for registered large types."""
        registry = AutoCompressRegistry()
        adapter = DateTimeAdapter()

        # Register datetime as a large type
        registry.register_large_type(datetime)

        wrapped = registry.wrap_adapter(adapter)
        assert isinstance(wrapped, CompressedAdapter)

    def test_wrap_adapter_disabled(self) -> None:
        """Test wrapping when compression is disabled."""
        config = CompressionConfig(enabled=False)
        registry = AutoCompressRegistry(config=config)
        adapter = DateTimeAdapter()

        # Register datetime as large but compression disabled
        registry.register_large_type(datetime)

        wrapped = registry.wrap_adapter(adapter)
        assert wrapped is adapter  # Should return as-is


# ============================================================================
# Extended FunctionWrapper Tests
# ============================================================================


class TestFunctionWrapperExtended:
    """Extended tests for FunctionWrapper to improve coverage."""

    def test_function_with_no_type_hints(self) -> None:
        """Test wrapping function without type hints."""

        def no_hints(x, y):
            return x + y

        wrapper = FunctionWrapper(no_hints)
        result = wrapper.call({"x": 1, "y": 2})
        assert result == 3

    def test_function_with_defaults(self) -> None:
        """Test wrapping function with default parameters."""

        def with_defaults(name: str, count: int = 5) -> str:
            return name * count

        wrapper = FunctionWrapper(with_defaults)
        schema = wrapper.get_json_schema()

        # name should be required, count should not
        assert "name" in schema["required"]
        assert "count" not in schema["required"]

    def test_get_return_schema_no_return_type(self) -> None:
        """Test getting return schema when no return type."""

        def no_return(x: int) -> None:
            pass

        wrapper = FunctionWrapper(no_return)
        schema = wrapper.get_return_schema()
        assert schema == {}

    def test_get_return_schema_with_return_type(self) -> None:
        """Test getting return schema with return type."""

        def with_return(x: int) -> str:
            return str(x)

        wrapper = FunctionWrapper(with_return)
        schema = wrapper.get_return_schema()
        assert "type" in schema

    def test_transform_input_no_info(self) -> None:
        """Test transform_input when parameter has no type info."""

        def func(x):
            return x

        wrapper = FunctionWrapper(func)
        result = wrapper.transform_input("x", 42)
        assert result == 42

    def test_transform_input_with_error(self) -> None:
        """Test transform_input raises TypeTransformError on failure."""
        registry = TypeRegistry()

        # Create a broken adapter
        adapter = FunctionAdapter(
            target_type=datetime,
            serializer=lambda dt: dt.isoformat(),
            deserializer=lambda s: datetime.fromisoformat(s),
            schema={"type": "string"},
        )
        registry.register_adapter(adapter)

        def func(d: datetime) -> str:
            return str(d)

        wrapper = FunctionWrapper(func, registry=registry)

        with pytest.raises(TypeTransformError):
            wrapper.transform_input("d", "not-a-date")

    def test_transform_output_none(self) -> None:
        """Test transform_output with None value."""

        def returns_none() -> None:
            return None

        wrapper = FunctionWrapper(returns_none)
        result = wrapper.transform_output(None)
        assert result is None

    def test_transform_output_with_adapter(self) -> None:
        """Test transform_output with adapter type."""
        registry = TypeRegistry()
        register_stdlib_adapters(registry)

        def get_date() -> datetime:
            return datetime(2024, 1, 15, 10, 30, 0)

        wrapper = FunctionWrapper(get_date, registry=registry)
        result = wrapper.transform_output(datetime(2024, 1, 15, 10, 30, 0))
        assert result == "2024-01-15T10:30:00"

    def test_transform_output_passthrough(self) -> None:
        """Test transform_output with passthrough type."""

        def get_dict() -> dict:
            return {"a": 1}

        wrapper = FunctionWrapper(get_dict)
        result = wrapper.transform_output({"a": 1})
        assert result == {"a": 1}

    def test_transform_output_object_store(self) -> None:
        """Test transform_output with object store type."""
        registry = TypeRegistry()
        store = ObjectStore()

        class Session:
            pass

        registry.register_stored_type(Session, ttl=3600)

        # Use wrapper's call method which handles the full flow
        def create_session() -> dict:
            return {"created": True}

        wrapper = FunctionWrapper(create_session, registry=registry, store=store)

        # Test _apply_output_transform directly with OBJECT_STORE strategy
        info = registry.get_type_info(Session)
        assert info.strategy == TypeStrategy.OBJECT_STORE

        session = Session()
        result = wrapper._apply_output_transform(info, session)

        # Should return a handle string
        assert isinstance(result, str)
        assert store.exists(result)

    def test_auto_serialize_output_list(self) -> None:
        """Test auto-serialization of lists."""

        def get_list():
            return [datetime(2024, 1, 15), datetime(2024, 1, 16)]

        registry = TypeRegistry()
        register_stdlib_adapters(registry)
        wrapper = FunctionWrapper(get_list, registry=registry)

        result = wrapper._auto_serialize_output([datetime(2024, 1, 15)])
        assert isinstance(result, list)
        # Should serialize datetime to string
        assert result[0] == "2024-01-15T00:00:00"

    def test_auto_serialize_output_dict(self) -> None:
        """Test auto-serialization of dicts."""
        registry = TypeRegistry()
        register_stdlib_adapters(registry)
        wrapper = FunctionWrapper(lambda: None, registry=registry)

        result = wrapper._auto_serialize_output({"date": datetime(2024, 1, 15)})
        assert result == {"date": "2024-01-15T00:00:00"}

    def test_auto_serialize_output_unknown_type(self) -> None:
        """Test auto-serialization of unknown types falls back to str."""

        class CustomType:
            def __str__(self) -> str:
                return "custom"

        wrapper = FunctionWrapper(lambda: None)
        result = wrapper._auto_serialize_output(CustomType())
        assert result == "custom"

    def test_call_with_transform_disabled(self) -> None:
        """Test call with input/output transforms disabled."""

        def add(a: int, b: int) -> int:
            return a + b

        wrapper = FunctionWrapper(add, transform_inputs=False, transform_output=False)
        result = wrapper.call({"a": 1, "b": 2})
        assert result == 3

    def test_direct_call(self) -> None:
        """Test direct __call__ method."""

        def multiply(a: int, b: int) -> int:
            return a * b

        wrapper = FunctionWrapper(multiply)
        result = wrapper(3, 4)
        assert result == 12

    def test_direct_call_with_type_transform(self) -> None:
        """Test direct __call__ with type transformation."""
        registry = TypeRegistry()
        register_stdlib_adapters(registry)

        def get_day(d: datetime) -> int:
            return d.day

        wrapper = FunctionWrapper(get_day, registry=registry)
        # Pass actual datetime object
        result = wrapper(datetime(2024, 1, 15))
        assert result == 15

    def test_maybe_serialize_input_json_types(self) -> None:
        """Test _maybe_serialize_input with JSON-compatible types."""

        def func(x: str) -> str:
            return x

        wrapper = FunctionWrapper(func)

        # All these should pass through
        assert wrapper._maybe_serialize_input("x", None) is None
        assert wrapper._maybe_serialize_input("x", True) is True
        assert wrapper._maybe_serialize_input("x", 42) == 42
        assert wrapper._maybe_serialize_input("x", 3.14) == 3.14
        assert wrapper._maybe_serialize_input("x", "hello") == "hello"
        assert wrapper._maybe_serialize_input("x", [1, 2]) == [1, 2]
        assert wrapper._maybe_serialize_input("x", {"a": 1}) == {"a": 1}

    def test_maybe_serialize_input_with_adapter(self) -> None:
        """Test _maybe_serialize_input with adapter types."""
        registry = TypeRegistry()
        register_stdlib_adapters(registry)

        def func(d: datetime) -> str:
            return str(d)

        wrapper = FunctionWrapper(func, registry=registry)

        # Pass actual datetime - should serialize it
        dt = datetime(2024, 1, 15)
        result = wrapper._maybe_serialize_input("d", dt)
        assert result == "2024-01-15T00:00:00"

    def test_maybe_serialize_input_no_info(self) -> None:
        """Test _maybe_serialize_input with no param info."""

        class CustomType:
            pass

        def func(x):
            return x

        wrapper = FunctionWrapper(func)
        obj = CustomType()
        result = wrapper._maybe_serialize_input("x", obj)
        assert result is obj


class TestAsyncFunctionWrapper:
    """Tests for async function wrapping."""

    @pytest.mark.asyncio
    async def test_call_async(self) -> None:
        """Test call_async method."""

        async def async_add(a: int, b: int) -> int:
            return a + b

        wrapper = FunctionWrapper(async_add)
        result = await wrapper.call_async({"a": 1, "b": 2})
        assert result == 3

    @pytest.mark.asyncio
    async def test_call_async_with_transforms(self) -> None:
        """Test call_async with type transforms."""
        registry = TypeRegistry()
        register_stdlib_adapters(registry)

        async def async_get_day(d: datetime) -> int:
            return d.day

        wrapper = FunctionWrapper(async_get_day, registry=registry)
        result = await wrapper.call_async({"d": "2024-01-15T10:30:00"})
        assert result == 15

    @pytest.mark.asyncio
    async def test_call_async_no_transform(self) -> None:
        """Test call_async with transforms disabled."""

        async def async_double(x: int) -> int:
            return x * 2

        wrapper = FunctionWrapper(
            async_double, transform_inputs=False, transform_output=False
        )
        result = await wrapper.call_async({"x": 5})
        assert result == 10


class TestAutoTransformDecorator:
    """Tests for the auto_transform decorator."""

    def test_auto_transform_basic(self) -> None:
        """Test basic auto_transform decorator usage."""
        from auto_mcp.types import auto_transform

        @auto_transform()
        def greet(name: str) -> str:
            return f"Hello, {name}!"

        result = greet.call({"name": "World"})
        assert result == "Hello, World!"

    def test_auto_transform_with_registry(self) -> None:
        """Test auto_transform with custom registry."""
        from auto_mcp.types import auto_transform

        registry = TypeRegistry()
        register_stdlib_adapters(registry)

        @auto_transform(registry=registry)
        def get_day(d: datetime) -> int:
            return d.day

        result = get_day.call({"d": "2024-01-15T10:30:00"})
        assert result == 15


# ============================================================================
# MethodWrapper and ClassWrapper Tests
# ============================================================================


class TestMethodWrapper:
    """Tests for MethodWrapper class."""

    def test_method_wrapper_basic(self) -> None:
        """Test basic method wrapping."""
        from auto_mcp.types.wrapper import MethodWrapper

        class Calculator:
            def add(self, a: int, b: int) -> int:
                return a + b

        calc = Calculator()
        wrapper = MethodWrapper(Calculator.add, calc)
        result = wrapper.call({"a": 1, "b": 2})
        assert result == 3

    def test_method_wrapper_with_registry(self) -> None:
        """Test method wrapper with type registry."""
        from auto_mcp.types.wrapper import MethodWrapper

        registry = TypeRegistry()
        register_stdlib_adapters(registry)

        class DateFormatter:
            def format_date(self, d: datetime) -> str:
                return d.strftime("%Y-%m-%d")

        formatter = DateFormatter()
        wrapper = MethodWrapper(DateFormatter.format_date, formatter, registry=registry)
        result = wrapper.call({"d": "2024-01-15T10:30:00"})
        assert result == "2024-01-15"


class TestClassWrapper:
    """Tests for ClassWrapper class."""

    def test_class_wrapper_basic(self) -> None:
        """Test basic class wrapping."""
        from auto_mcp.types.wrapper import ClassWrapper

        class Calculator:
            def add(self, a: int, b: int) -> int:
                return a + b

            def multiply(self, a: int, b: int) -> int:
                return a * b

        wrapper = ClassWrapper(Calculator)
        instance = wrapper.create()

        result = wrapper.call_method(instance, "add", {"a": 3, "b": 4})
        assert result == 7

        result = wrapper.call_method(instance, "multiply", {"a": 3, "b": 4})
        assert result == 12

    def test_class_wrapper_get_method_names(self) -> None:
        """Test getting method names from class wrapper."""
        from auto_mcp.types.wrapper import ClassWrapper

        class MyClass:
            def public_method(self) -> str:
                return "public"

            def _private_method(self) -> str:
                return "private"

        wrapper = ClassWrapper(MyClass)
        names = wrapper.get_method_names()

        assert "public_method" in names
        assert "_private_method" not in names

    def test_class_wrapper_get_method_schema(self) -> None:
        """Test getting method schema from class wrapper."""
        from auto_mcp.types.wrapper import ClassWrapper

        class MyClass:
            def process(self, name: str, count: int) -> dict:
                return {"name": name, "count": count}

        wrapper = ClassWrapper(MyClass)
        schema = wrapper.get_method_schema("process")

        assert "properties" in schema
        assert "name" in schema["properties"]
        assert "count" in schema["properties"]

    def test_class_wrapper_method_not_found(self) -> None:
        """Test error when method not found."""
        from auto_mcp.types.wrapper import ClassWrapper

        class MyClass:
            def exists(self) -> str:
                return "yes"

        wrapper = ClassWrapper(MyClass)

        with pytest.raises(KeyError, match="Method not found"):
            wrapper.get_method_schema("nonexistent")

        with pytest.raises(KeyError, match="Method not found"):
            wrapper.call_method(wrapper.create(), "nonexistent", {})

    def test_class_wrapper_create_with_init_kwargs(self) -> None:
        """Test creating instance with init kwargs."""
        from auto_mcp.types.wrapper import ClassWrapper

        class Greeter:
            def __init__(self, prefix: str) -> None:
                self.prefix = prefix

            def greet(self, name: str) -> str:
                return f"{self.prefix} {name}"

        wrapper = ClassWrapper(Greeter)
        instance = wrapper.create({"prefix": "Hello"})

        result = wrapper.call_method(instance, "greet", {"name": "World"})
        assert result == "Hello World"

    def test_class_wrapper_with_type_transforms(self) -> None:
        """Test class wrapper with type transformations."""
        from auto_mcp.types.wrapper import ClassWrapper

        registry = TypeRegistry()
        register_stdlib_adapters(registry)

        class DateProcessor:
            def get_year(self, d: datetime) -> int:
                return d.year

        wrapper = ClassWrapper(DateProcessor, registry=registry)
        instance = wrapper.create()

        result = wrapper.call_method(instance, "get_year", {"d": "2024-01-15T10:30:00"})
        assert result == 2024


class TestFunctionWrapperSessionInjection:
    """Tests for FunctionWrapper session injection."""

    @pytest.mark.asyncio
    async def test_call_with_session_injection(self) -> None:
        """Test FunctionWrapper.call() with session injection."""
        from auto_mcp.session.context import SessionContext
        from auto_mcp.session.manager import SessionManager
        from auto_mcp.types.wrapper import FunctionWrapper

        manager = SessionManager()
        session = await manager.create_session()

        def session_aware_func(session: SessionContext, x: int) -> str:
            return f"session:{session.session_id}, x:{x}"

        wrapper = FunctionWrapper(
            session_aware_func,
            session_manager=manager,
            session_param_name="session",
        )

        result = wrapper.call({"session_id": session.session_id, "x": 42})
        assert f"session:{session.session_id}" in result
        assert "x:42" in result

    @pytest.mark.asyncio
    async def test_call_with_session_missing_session_id(self) -> None:
        """Test FunctionWrapper.call() raises error when session_id is missing."""
        from auto_mcp.session.context import SessionContext
        from auto_mcp.session.manager import SessionManager
        from auto_mcp.types.wrapper import FunctionWrapper

        manager = SessionManager()

        def session_aware_func(session: SessionContext, x: int) -> str:
            return "result"

        wrapper = FunctionWrapper(
            session_aware_func,
            session_manager=manager,
            session_param_name="session",
        )

        with pytest.raises(ValueError, match="session_id is required"):
            wrapper.call({"x": 42})

    @pytest.mark.asyncio
    async def test_call_with_session_invalid_session_id_type(self) -> None:
        """Test FunctionWrapper.call() raises error for non-string session_id."""
        from auto_mcp.session.context import SessionContext
        from auto_mcp.session.manager import SessionManager
        from auto_mcp.types.wrapper import FunctionWrapper

        manager = SessionManager()

        def session_aware_func(session: SessionContext, x: int) -> str:
            return "result"

        wrapper = FunctionWrapper(
            session_aware_func,
            session_manager=manager,
            session_param_name="session",
        )

        with pytest.raises(TypeError, match="session_id must be a string"):
            wrapper.call({"session_id": 123, "x": 42})

    @pytest.mark.asyncio
    async def test_call_async_with_session_injection(self) -> None:
        """Test FunctionWrapper.call_async() with session injection."""
        from auto_mcp.session.context import SessionContext
        from auto_mcp.session.manager import SessionManager
        from auto_mcp.types.wrapper import FunctionWrapper

        manager = SessionManager()
        session = await manager.create_session()

        async def async_session_func(session: SessionContext, x: int) -> str:
            return f"async session:{session.session_id}, x:{x}"

        wrapper = FunctionWrapper(
            async_session_func,
            session_manager=manager,
            session_param_name="session",
        )

        result = await wrapper.call_async({"session_id": session.session_id, "x": 99})
        assert f"session:{session.session_id}" in result
        assert "x:99" in result

    @pytest.mark.asyncio
    async def test_call_async_missing_session_id(self) -> None:
        """Test FunctionWrapper.call_async() raises error when session_id is missing."""
        from auto_mcp.session.context import SessionContext
        from auto_mcp.session.manager import SessionManager
        from auto_mcp.types.wrapper import FunctionWrapper

        manager = SessionManager()

        async def async_session_func(session: SessionContext, x: int) -> str:
            return "result"

        wrapper = FunctionWrapper(
            async_session_func,
            session_manager=manager,
            session_param_name="session",
        )

        with pytest.raises(ValueError, match="session_id is required"):
            await wrapper.call_async({"x": 42})

    @pytest.mark.asyncio
    async def test_call_async_invalid_session_id_type(self) -> None:
        """Test FunctionWrapper.call_async() raises error for non-string session_id."""
        from auto_mcp.session.context import SessionContext
        from auto_mcp.session.manager import SessionManager
        from auto_mcp.types.wrapper import FunctionWrapper

        manager = SessionManager()

        async def async_session_func(session: SessionContext, x: int) -> str:
            return "result"

        wrapper = FunctionWrapper(
            async_session_func,
            session_manager=manager,
            session_param_name="session",
        )

        with pytest.raises(TypeError, match="session_id must be a string"):
            await wrapper.call_async({"session_id": 123, "x": 42})


class TestFunctionWrapperObjectStore:
    """Tests for FunctionWrapper with object store strategy."""

    def test_object_store_handle_type_error(self) -> None:
        """Test object store raises TypeError for non-string handle."""
        from auto_mcp.types.base import TypeInfo, TypeStrategy
        from auto_mcp.types.wrapper import FunctionWrapper

        def func_with_object(obj: dict) -> str:
            return str(obj)

        wrapper = FunctionWrapper(func_with_object)

        # Create a TypeInfo with OBJECT_STORE strategy
        info = TypeInfo(
            type_=dict,
            strategy=TypeStrategy.OBJECT_STORE,
        )

        # This should raise TypeError for non-string handle
        with pytest.raises(TypeError, match="Expected handle string"):
            wrapper._apply_input_transform(info, 123)


class TestFunctionWrapperOutputTransform:
    """Tests for FunctionWrapper output transformation edge cases."""

    def test_output_transform_failure_fallback(self) -> None:
        """Test output transformation fallback on failure."""
        from auto_mcp.types.base import TypeInfo, TypeStrategy
        from auto_mcp.types.wrapper import FunctionWrapper

        # A function that returns a custom type
        class CustomType:
            def __init__(self, value: int) -> None:
                self.value = value

            def __str__(self) -> str:
                return f"CustomType({self.value})"

        def func_with_custom_return() -> CustomType:
            return CustomType(42)

        # Wrapper without adapter for CustomType
        wrapper = FunctionWrapper(func_with_custom_return)

        # Should fall back to auto-serialization
        result = wrapper.call({})
        # The auto-serialization should have converted it somehow
        assert result is not None


# ============================================================================
# Extended Adapter Tests
# ============================================================================


class TestAdapterErrorHandling:
    """Tests for adapter error handling."""

    def test_datetime_adapter_type_error(self) -> None:
        """Test DateTimeAdapter raises TypeError for non-string input."""
        adapter = DateTimeAdapter()
        with pytest.raises(TypeError, match="Expected string"):
            adapter.deserialize(123)

    def test_date_adapter_type_error(self) -> None:
        """Test DateAdapter raises TypeError for non-string input."""
        adapter = DateAdapter()
        with pytest.raises(TypeError, match="Expected string"):
            adapter.deserialize(123)

    def test_time_adapter_type_error(self) -> None:
        """Test TimeAdapter raises TypeError for non-string input."""
        adapter = TimeAdapter()
        with pytest.raises(TypeError, match="Expected string"):
            adapter.deserialize(123)

    def test_timedelta_adapter_type_error(self) -> None:
        """Test TimeDeltaAdapter raises TypeError for non-number input."""
        adapter = TimeDeltaAdapter()
        with pytest.raises(TypeError, match="Expected number"):
            adapter.deserialize("not a number")

    def test_path_adapter_type_error(self) -> None:
        """Test PathAdapter raises TypeError for non-string input."""
        adapter = PathAdapter()
        with pytest.raises(TypeError, match="Expected string"):
            adapter.deserialize(123)

    def test_purepath_adapter_type_error(self) -> None:
        """Test PurePathAdapter raises TypeError for non-string input."""
        adapter = PurePathAdapter()
        with pytest.raises(TypeError, match="Expected string"):
            adapter.deserialize(123)

    def test_uuid_adapter_type_error(self) -> None:
        """Test UUIDAdapter raises TypeError for non-string input."""
        adapter = UUIDAdapter()
        with pytest.raises(TypeError, match="Expected string"):
            adapter.deserialize(123)

    def test_decimal_adapter_type_error(self) -> None:
        """Test DecimalAdapter raises TypeError for unsupported input."""
        adapter = DecimalAdapter()
        with pytest.raises(TypeError, match="Expected string or number"):
            adapter.deserialize([1, 2, 3])

    def test_decimal_adapter_from_int(self) -> None:
        """Test DecimalAdapter deserializes from int."""
        adapter = DecimalAdapter()
        result = adapter.deserialize(42)
        assert result == Decimal("42")

    def test_decimal_adapter_from_float(self) -> None:
        """Test DecimalAdapter deserializes from float."""
        adapter = DecimalAdapter()
        result = adapter.deserialize(3.14)
        assert result == Decimal("3.14")

    def test_bytes_adapter_type_error(self) -> None:
        """Test BytesAdapter raises TypeError for non-string input."""
        adapter = BytesAdapter()
        with pytest.raises(TypeError, match="Expected string"):
            adapter.deserialize(123)

    def test_bytearray_adapter_type_error(self) -> None:
        """Test ByteArrayAdapter raises TypeError for non-string input."""
        adapter = ByteArrayAdapter()
        with pytest.raises(TypeError, match="Expected string"):
            adapter.deserialize(123)

    def test_set_adapter_type_error(self) -> None:
        """Test SetAdapter raises TypeError for non-list input."""
        adapter = SetAdapter()
        with pytest.raises(TypeError, match="Expected list"):
            adapter.deserialize("not a list")

    def test_frozenset_adapter_type_error(self) -> None:
        """Test FrozenSetAdapter raises TypeError for non-list input."""
        adapter = FrozenSetAdapter()
        with pytest.raises(TypeError, match="Expected list"):
            adapter.deserialize("not a list")

    def test_complex_adapter_type_error(self) -> None:
        """Test ComplexAdapter raises TypeError for unsupported input."""
        adapter = ComplexAdapter()
        with pytest.raises(TypeError, match="Expected dict or string"):
            adapter.deserialize(123)

    def test_complex_adapter_from_string(self) -> None:
        """Test ComplexAdapter deserializes from string."""
        adapter = ComplexAdapter()
        result = adapter.deserialize("3+4j")
        assert result == complex(3, 4)


class TestSetAdapterUncomparable:
    """Tests for set adapters with uncomparable elements."""

    def test_set_adapter_uncomparable(self) -> None:
        """Test SetAdapter with uncomparable elements."""
        adapter = SetAdapter()
        # Dict items are uncomparable
        s = {frozenset([1]), frozenset([2])}
        serialized = adapter.serialize(s)
        # Should fallback to list conversion
        assert isinstance(serialized, list)

    def test_frozenset_adapter_uncomparable(self) -> None:
        """Test FrozenSetAdapter with uncomparable elements."""
        adapter = FrozenSetAdapter()
        fs = frozenset([frozenset([1]), frozenset([2])])
        serialized = adapter.serialize(fs)
        assert isinstance(serialized, list)


class TestBytesAdapterConstraints:
    """Tests for bytes adapter constraints."""

    def test_bytes_adapter_can_serialize_wrong_type(self) -> None:
        """Test BytesAdapter can_serialize with wrong type."""
        adapter = BytesAdapter()
        assert adapter.can_serialize("not bytes") is False  # type: ignore

    def test_bytes_adapter_no_limit(self) -> None:
        """Test BytesAdapter with no size limit."""
        adapter = BytesAdapter()
        large_bytes = b"x" * 10000
        assert adapter.can_serialize(large_bytes) is True

    def test_bytearray_adapter_can_serialize_wrong_type(self) -> None:
        """Test ByteArrayAdapter can_serialize with wrong type."""
        adapter = ByteArrayAdapter()
        assert adapter.can_serialize("not bytearray") is False  # type: ignore

    def test_bytearray_adapter_max_size(self) -> None:
        """Test ByteArrayAdapter max size constraint."""
        adapter = ByteArrayAdapter(max_size=10)
        assert adapter.can_serialize(bytearray(b"short")) is True
        assert adapter.can_serialize(bytearray(b"x" * 20)) is False


# ============================================================================
# Extended Registry Tests
# ============================================================================


class TestTypeRegistryExtended:
    """Extended tests for TypeRegistry."""

    def test_get_strategy(self) -> None:
        """Test get_strategy method."""
        registry = TypeRegistry()
        registry.register_adapter(DateTimeAdapter())

        assert registry.get_strategy(int) == TypeStrategy.PASSTHROUGH
        assert registry.get_strategy(datetime) == TypeStrategy.ADAPTER

    def test_serialize_passthrough(self) -> None:
        """Test serializing passthrough types."""
        registry = TypeRegistry()
        result = registry.serialize("hello")
        assert result == "hello"

    def test_serialize_object_store_error(self) -> None:
        """Test serialize raises error for object store types."""
        registry = TypeRegistry()

        class Session:
            pass

        registry.register_stored_type(Session)

        with pytest.raises(TypeError, match="uses object store strategy"):
            registry.serialize(Session())

    def test_serialize_unsupported_error(self) -> None:
        """Test serialize raises error for unsupported types."""
        registry = TypeRegistry()

        class Unsupported:
            pass

        with pytest.raises(TypeError, match="No serialization strategy"):
            registry.serialize(Unsupported())

    def test_serialize_adapter_cannot_serialize(self) -> None:
        """Test serialize raises error when adapter can't serialize."""
        registry = TypeRegistry()
        adapter = BytesAdapter(max_size=10)
        registry.register_adapter(adapter)

        with pytest.raises(ValueError, match="cannot serialize"):
            registry.serialize(b"x" * 100)

    def test_deserialize_passthrough(self) -> None:
        """Test deserializing passthrough types."""
        registry = TypeRegistry()
        result = registry.deserialize("hello", str)
        assert result == "hello"

    def test_deserialize_object_store_error(self) -> None:
        """Test deserialize raises error for object store types."""
        registry = TypeRegistry()

        class Session:
            pass

        registry.register_stored_type(Session)

        with pytest.raises(TypeError, match="uses object store strategy"):
            registry.deserialize("handle:123", Session)

    def test_deserialize_unsupported_error(self) -> None:
        """Test deserialize raises error for unsupported types."""
        registry = TypeRegistry()

        class Unsupported:
            pass

        with pytest.raises(TypeError, match="No deserialization strategy"):
            registry.deserialize({}, Unsupported)

    def test_parent_class_adapter_lookup(self) -> None:
        """Test adapter lookup for parent classes."""
        registry = TypeRegistry()

        class Base:
            pass

        class Derived(Base):
            pass

        adapter = FunctionAdapter(
            target_type=Base,
            serializer=lambda x: "base",
            deserializer=lambda s: Base(),
            schema={"type": "string"},
        )
        registry.register_adapter(adapter)

        # Should find adapter via parent class
        found = registry.get_adapter(Derived)
        assert found is adapter

    def test_parent_class_store_config_lookup(self) -> None:
        """Test store config lookup for parent classes."""
        registry = TypeRegistry()

        class BaseSession:
            pass

        class DerivedSession(BaseSession):
            pass

        registry.register_stored_type(BaseSession, ttl=7200)

        # Should find config via parent class
        config = registry.get_store_config(DerivedSession)
        assert config is not None
        assert config.ttl == 7200

    def test_overwrite_warning(self) -> None:
        """Test that overwriting adapter logs warning."""
        registry = TypeRegistry()
        registry.register_adapter(DateTimeAdapter())
        # Second registration should overwrite without error
        registry.register_adapter(DateTimeAdapter())
        # Should still work
        assert registry.has_adapter(datetime)

    def test_get_type_info_unsupported(self) -> None:
        """Test get_type_info for unsupported types."""
        registry = TypeRegistry()

        class Unsupported:
            pass

        info = registry.get_type_info(Unsupported)
        assert info.strategy == TypeStrategy.UNSUPPORTED
        assert info.json_schema == {}

    def test_primitive_schema_all_types(self) -> None:
        """Test _get_primitive_schema for all primitive types."""
        registry = TypeRegistry()

        assert registry._get_primitive_schema(type(None)) == {"type": "null"}
        assert registry._get_primitive_schema(bool) == {"type": "boolean"}
        assert registry._get_primitive_schema(int) == {"type": "integer"}
        assert registry._get_primitive_schema(float) == {"type": "number"}
        assert registry._get_primitive_schema(str) == {"type": "string"}
        assert registry._get_primitive_schema(list) == {"type": "array"}
        assert registry._get_primitive_schema(dict) == {"type": "object"}

    def test_primitive_schema_unknown(self) -> None:
        """Test _get_primitive_schema for unknown types."""
        registry = TypeRegistry()

        class Unknown:
            pass

        assert registry._get_primitive_schema(Unknown) == {}


# ============================================================================
# Extended Base Types Tests
# ============================================================================


class TestBaseTypesExtended:
    """Extended tests for base types."""

    def test_type_info_type_name_builtin(self) -> None:
        """Test type_name for builtin types."""
        info = TypeInfo(type_=int, strategy=TypeStrategy.PASSTHROUGH)
        assert info.type_name == "int"

    def test_type_info_type_name_module(self) -> None:
        """Test type_name for module types."""
        info = TypeInfo(type_=datetime, strategy=TypeStrategy.ADAPTER)
        assert "datetime" in info.type_name

    def test_is_json_compatible_generic_list(self) -> None:
        """Test is_json_compatible with generic list."""
        from typing import List

        assert is_json_compatible(List[int]) is True  # type: ignore

    def test_is_json_compatible_generic_dict(self) -> None:
        """Test is_json_compatible with generic dict."""
        from typing import Dict

        assert is_json_compatible(Dict[str, Any]) is True  # type: ignore

    def test_is_json_compatible_other_generic(self) -> None:
        """Test is_json_compatible with non-list/dict generic."""
        from typing import Optional

        assert is_json_compatible(Optional[int]) is False

    def test_get_type_origin(self) -> None:
        """Test get_type_origin utility function."""
        from typing import List

        from auto_mcp.types.base import get_type_origin

        assert get_type_origin(List[int]) is list  # type: ignore
        assert get_type_origin(int) is None

    def test_get_type_args(self) -> None:
        """Test get_type_args utility function."""
        from typing import Dict

        from auto_mcp.types.base import get_type_args

        args = get_type_args(Dict[str, int])  # type: ignore
        assert str in args
        assert int in args

        # No args for non-generic
        assert get_type_args(int) == ()


class TestTypeAdapterDecorator:
    """Tests for the type_adapter decorator edge cases."""

    def test_type_adapter_missing_serialize(self) -> None:
        """Test type_adapter raises error for missing serialize."""
        from auto_mcp.types.base import type_adapter

        with pytest.raises(TypeError, match="must implement serialize"):

            @type_adapter(datetime)
            class MissingSerialize:
                def deserialize(self, data: str) -> datetime:
                    return datetime.fromisoformat(data)

                def json_schema(self) -> dict:
                    return {}

    def test_type_adapter_missing_deserialize(self) -> None:
        """Test type_adapter raises error for missing deserialize."""
        from auto_mcp.types.base import type_adapter

        with pytest.raises(TypeError, match="must implement deserialize"):

            @type_adapter(datetime)
            class MissingDeserialize:
                def serialize(self, obj: datetime) -> str:
                    return obj.isoformat()

                def json_schema(self) -> dict:
                    return {}

    def test_type_adapter_missing_json_schema(self) -> None:
        """Test type_adapter raises error for missing json_schema."""
        from auto_mcp.types.base import type_adapter

        with pytest.raises(TypeError, match="must implement json_schema"):

            @type_adapter(datetime)
            class MissingSchema:
                def serialize(self, obj: datetime) -> str:
                    return obj.isoformat()

                def deserialize(self, data: str) -> datetime:
                    return datetime.fromisoformat(data)


class TestTypeAdapterValidate:
    """Tests for TypeAdapter validate method."""

    def test_validate_default(self) -> None:
        """Test default validate always returns True."""
        adapter = DateTimeAdapter()
        assert adapter.validate("2024-01-15T10:30:00") is True
        assert adapter.validate(123) is True  # Default doesn't validate


# ============================================================================
# Extended Compression Tests
# ============================================================================


class TestCompressionExtended:
    """Extended tests for compression module."""

    def test_compressed_data_zero_original_size(self) -> None:
        """Test CompressedData with zero original size."""
        data = CompressedData(
            compressed=False,
            algorithm="none",
            original_size=0,
            compressed_size=0,
            data="",
        )
        d = data.to_dict()
        assert d["compression_ratio"] == 1.0

    def test_decompress_none(self) -> None:
        """Test decompression with NONE algorithm."""
        original = b"Hello"
        result = decompress_bytes(original, CompressionAlgorithm.NONE)
        assert result == original

    def test_compressed_adapter_compression_not_worth_it(self) -> None:
        """Test CompressedAdapter when compression doesn't save space."""
        # Create adapter for small incompressible data
        adapter = FunctionAdapter(
            target_type=str,
            serializer=lambda s: s,
            deserializer=lambda s: s,
            schema={"type": "string"},
        )
        config = CompressionConfig(threshold=1)  # Force compression attempt
        compressed = CompressedAdapter(adapter, config)

        # Very small string - compression may not help
        result = compressed.serialize("x")
        assert isinstance(result, dict)
        # Either compressed or not, it should work
        assert "compressed" in result

    def test_compressed_adapter_disabled(self) -> None:
        """Test CompressedAdapter when compression is disabled."""
        adapter = DateTimeAdapter()
        config = CompressionConfig(enabled=False)
        compressed = CompressedAdapter(adapter, config)

        dt = datetime(2024, 1, 15, 10, 30, 0)
        result = compressed.serialize(dt)

        assert isinstance(result, dict)
        assert result["compressed"] is False

    def test_compress_zlib_different_level(self) -> None:
        """Test zlib compression with different levels."""
        data = b"Hello World! " * 100

        # Level 1 (fast)
        compressed_fast = compress_bytes(data, CompressionAlgorithm.ZLIB, level=1)
        # Level 9 (best)
        compressed_best = compress_bytes(data, CompressionAlgorithm.ZLIB, level=9)

        # Both should decompress correctly
        assert decompress_bytes(compressed_fast, CompressionAlgorithm.ZLIB) == data
        assert decompress_bytes(compressed_best, CompressionAlgorithm.ZLIB) == data

    def test_with_compression_all_params(self) -> None:
        """Test with_compression with all parameters."""
        adapter = DateTimeAdapter()
        compressed = with_compression(
            adapter,
            threshold=2048,
            algorithm=CompressionAlgorithm.ZLIB,
            level=9,
        )

        assert isinstance(compressed, CompressedAdapter)
        assert compressed.config.threshold == 2048
        assert compressed.config.algorithm == CompressionAlgorithm.ZLIB
        assert compressed.config.level == 9


# ============================================================================
# Object Store Extended Tests
# ============================================================================


# ============================================================================
# Optional Third-Party Adapter Tests
# ============================================================================


class TestPandasDataFrameAdapter:
    """Tests for pandas DataFrame adapter."""

    def test_create_adapter(self) -> None:
        """Test creating pandas DataFrame adapter."""
        from auto_mcp.types.adapters import create_pandas_dataframe_adapter

        adapter = create_pandas_dataframe_adapter()
        assert adapter is not None

    def test_serialize_dataframe(self) -> None:
        """Test serializing a DataFrame."""
        import pandas as pd

        from auto_mcp.types.adapters import create_pandas_dataframe_adapter

        adapter = create_pandas_dataframe_adapter()
        assert adapter is not None

        df = pd.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]})
        result = adapter.serialize(df)

        assert isinstance(result, dict)
        assert result["columns"] == ["a", "b"]
        assert result["shape"] == [3, 2]
        assert result["truncated"] is False

    def test_deserialize_dataframe(self) -> None:
        """Test deserializing a DataFrame."""
        import pandas as pd

        from auto_mcp.types.adapters import create_pandas_dataframe_adapter

        adapter = create_pandas_dataframe_adapter()
        assert adapter is not None

        data = {
            "columns": ["a", "b"],
            "data": [[1, 4], [2, 5], [3, 6]],
        }
        df = adapter.deserialize(data)

        assert isinstance(df, pd.DataFrame)
        assert list(df.columns) == ["a", "b"]
        assert len(df) == 3

    def test_dataframe_truncation(self) -> None:
        """Test DataFrame truncation for large data."""
        import pandas as pd

        from auto_mcp.types.adapters import create_pandas_dataframe_adapter

        # Create adapter with low max_rows
        adapter = create_pandas_dataframe_adapter()
        assert adapter is not None

        # Manually set max_rows lower for testing
        adapter._max_rows = 2

        df = pd.DataFrame({"a": range(10)})
        result = adapter.serialize(df)

        assert result["truncated"] is True
        assert len(result["data"]) == 2

    def test_dataframe_json_schema(self) -> None:
        """Test DataFrame JSON schema."""
        from auto_mcp.types.adapters import create_pandas_dataframe_adapter

        adapter = create_pandas_dataframe_adapter()
        assert adapter is not None

        schema = adapter.json_schema()
        assert schema["type"] == "object"
        assert "columns" in schema["properties"]
        assert "data" in schema["properties"]

    def test_dataframe_can_serialize(self) -> None:
        """Test DataFrame can_serialize."""
        import pandas as pd

        from auto_mcp.types.adapters import create_pandas_dataframe_adapter

        adapter = create_pandas_dataframe_adapter()
        assert adapter is not None

        df = pd.DataFrame({"a": [1]})
        assert adapter.can_serialize(df) is True
        assert adapter.can_serialize("not a dataframe") is False

    def test_deserialize_type_error(self) -> None:
        """Test DataFrame deserialize with wrong type."""
        from auto_mcp.types.adapters import create_pandas_dataframe_adapter

        adapter = create_pandas_dataframe_adapter()
        assert adapter is not None

        with pytest.raises(TypeError, match="Expected dict"):
            adapter.deserialize("not a dict")


class TestPILImageAdapter:
    """Tests for PIL Image adapter."""

    def test_create_adapter(self) -> None:
        """Test creating PIL Image adapter."""
        from auto_mcp.types.adapters import create_pil_image_adapter

        adapter = create_pil_image_adapter()
        assert adapter is not None

    def test_serialize_image(self) -> None:
        """Test serializing an image."""
        from PIL import Image

        from auto_mcp.types.adapters import create_pil_image_adapter

        adapter = create_pil_image_adapter()
        assert adapter is not None

        # Create a simple test image
        img = Image.new("RGB", (100, 100), color="red")
        result = adapter.serialize(img)

        assert isinstance(result, dict)
        assert "data" in result
        assert result["format"] == "png"
        assert result["width"] == 100
        assert result["height"] == 100
        assert result["mode"] == "RGB"

    def test_deserialize_image(self) -> None:
        """Test deserializing an image."""
        from PIL import Image

        from auto_mcp.types.adapters import create_pil_image_adapter

        adapter = create_pil_image_adapter()
        assert adapter is not None

        # Create and serialize an image first
        original = Image.new("RGB", (50, 50), color="blue")
        serialized = adapter.serialize(original)

        # Deserialize
        restored = adapter.deserialize(serialized)

        assert isinstance(restored, Image.Image)
        assert restored.size == (50, 50)

    def test_image_with_max_size(self) -> None:
        """Test image resizing with max_size."""
        from PIL import Image

        from auto_mcp.types.adapters import create_pil_image_adapter

        # Create adapter with max_size (we need to create it directly)
        adapter = create_pil_image_adapter()
        assert adapter is not None

        # Set max_size
        adapter._max_size = (50, 50)

        # Create large image
        img = Image.new("RGB", (200, 200), color="green")
        result = adapter.serialize(img)

        # Should be resized
        assert result["width"] <= 50
        assert result["height"] <= 50

    def test_image_json_schema(self) -> None:
        """Test Image JSON schema."""
        from auto_mcp.types.adapters import create_pil_image_adapter

        adapter = create_pil_image_adapter()
        assert adapter is not None

        schema = adapter.json_schema()
        assert schema["type"] == "object"
        assert "data" in schema["properties"]
        assert "format" in schema["properties"]

    def test_image_can_serialize(self) -> None:
        """Test Image can_serialize."""
        from PIL import Image

        from auto_mcp.types.adapters import create_pil_image_adapter

        adapter = create_pil_image_adapter()
        assert adapter is not None

        img = Image.new("RGB", (10, 10))
        assert adapter.can_serialize(img) is True
        assert adapter.can_serialize("not an image") is False

    def test_deserialize_type_error(self) -> None:
        """Test Image deserialize with wrong type."""
        from auto_mcp.types.adapters import create_pil_image_adapter

        adapter = create_pil_image_adapter()
        assert adapter is not None

        with pytest.raises(TypeError, match="Expected dict"):
            adapter.deserialize("not a dict")


class TestNumpyArrayAdapter:
    """Tests for numpy ndarray adapter."""

    def test_create_adapter(self) -> None:
        """Test creating numpy ndarray adapter."""
        from auto_mcp.types.adapters import create_numpy_array_adapter

        adapter = create_numpy_array_adapter()
        assert adapter is not None

    def test_serialize_array(self) -> None:
        """Test serializing a numpy array."""
        import numpy as np

        from auto_mcp.types.adapters import create_numpy_array_adapter

        adapter = create_numpy_array_adapter()
        assert adapter is not None

        arr = np.array([[1, 2, 3], [4, 5, 6]])
        result = adapter.serialize(arr)

        assert isinstance(result, dict)
        assert result["shape"] == [2, 3]
        assert result["dtype"] == "int64"
        assert result["truncated"] is False
        assert len(result["data"]) == 6

    def test_deserialize_array(self) -> None:
        """Test deserializing a numpy array."""
        import numpy as np

        from auto_mcp.types.adapters import create_numpy_array_adapter

        adapter = create_numpy_array_adapter()
        assert adapter is not None

        data = {
            "data": [1, 2, 3, 4, 5, 6],
            "shape": [2, 3],
            "dtype": "int64",
            "truncated": False,
        }
        arr = adapter.deserialize(data)

        assert isinstance(arr, np.ndarray)
        assert arr.shape == (2, 3)

    def test_array_truncation(self) -> None:
        """Test array truncation for large data."""
        import numpy as np

        from auto_mcp.types.adapters import create_numpy_array_adapter

        adapter = create_numpy_array_adapter()
        assert adapter is not None

        # Set low max_elements
        adapter._max_elements = 5

        arr = np.arange(100)
        result = adapter.serialize(arr)

        assert result["truncated"] is True
        assert len(result["data"]) == 5

    def test_deserialize_truncated_array(self) -> None:
        """Test deserializing truncated array doesn't reshape."""
        import numpy as np

        from auto_mcp.types.adapters import create_numpy_array_adapter

        adapter = create_numpy_array_adapter()
        assert adapter is not None

        data = {
            "data": [1, 2, 3, 4, 5],  # Only 5 elements
            "shape": [10, 10],  # Original was 100 elements
            "dtype": "int64",
            "truncated": True,  # Was truncated
        }
        arr = adapter.deserialize(data)

        # Should not reshape since truncated
        assert arr.shape == (5,)

    def test_array_json_schema(self) -> None:
        """Test ndarray JSON schema."""
        from auto_mcp.types.adapters import create_numpy_array_adapter

        adapter = create_numpy_array_adapter()
        assert adapter is not None

        schema = adapter.json_schema()
        assert schema["type"] == "object"
        assert "data" in schema["properties"]
        assert "shape" in schema["properties"]
        assert "dtype" in schema["properties"]

    def test_array_can_serialize(self) -> None:
        """Test ndarray can_serialize."""
        import numpy as np

        from auto_mcp.types.adapters import create_numpy_array_adapter

        adapter = create_numpy_array_adapter()
        assert adapter is not None

        arr = np.array([1, 2, 3])
        assert adapter.can_serialize(arr) is True
        assert adapter.can_serialize([1, 2, 3]) is False

    def test_deserialize_type_error(self) -> None:
        """Test ndarray deserialize with wrong type."""
        from auto_mcp.types.adapters import create_numpy_array_adapter

        adapter = create_numpy_array_adapter()
        assert adapter is not None

        with pytest.raises(TypeError, match="Expected dict"):
            adapter.deserialize("not a dict")


class TestOptionalAdaptersHelpers:
    """Tests for optional adapter helper functions."""

    def test_get_optional_adapters(self) -> None:
        """Test getting optional adapters."""
        from auto_mcp.types.adapters import get_optional_adapters

        adapters = get_optional_adapters()
        # Should have at least pandas, PIL, numpy adapters
        assert len(adapters) >= 3


# ============================================================================
# Object Store Extended Tests
# ============================================================================


class TestObjectStoreExtended:
    """Extended tests for ObjectStore."""

    def test_get_info_nonexistent(self) -> None:
        """Test get_info for non-existent handle."""
        store = ObjectStore()
        with pytest.raises(KeyError):
            store.get_info("nonexistent")

    def test_store_with_no_expiration(self) -> None:
        """Test storing with no expiration (ttl=0)."""
        store = ObjectStore()
        handle = store.store("test", ttl=0)

        info = store.get_info(handle)
        assert info["ttl_remaining"] is None

    def test_list_handles_by_type(self) -> None:
        """Test listing handles filtered by type."""
        store = ObjectStore()
        store.store("string1")
        store.store("string2")
        store.store(123)

        str_handles = store.list_handles(type_filter=str)
        int_handles = store.list_handles(type_filter=int)

        assert len(str_handles) == 2
        assert len(int_handles) == 1

    def test_count_by_type(self) -> None:
        """Test counting objects by type."""
        store = ObjectStore()
        store.store("string1")
        store.store("string2")
        store.store(123)

        assert store.count(type_filter=str) == 2
        assert store.count(type_filter=int) == 1

    def test_cleanup_with_auto_cleanup_disabled(self) -> None:
        """Test manual cleanup when auto_cleanup is disabled."""
        store = ObjectStore(auto_cleanup=False)

        # Store objects that will expire
        h1 = store.store("test1", ttl=3600)
        h2 = store.store("test2", ttl=3600)

        # Manually set one as expired
        store._objects[h1].expires_at = time.time() - 1

        # Cleanup should remove only expired
        removed = store.cleanup()
        assert removed == 1
        assert not store.exists(h1)
        assert store.exists(h2)



"""Tests for LLM providers."""

from __future__ import annotations

import inspect
from unittest.mock import AsyncMock, MagicMock, patch

import pytest

from auto_mcp.core.analyzer import MethodMetadata
from auto_mcp.llm import (
    AnthropicProvider,
    LLMProvider,
    OllamaProvider,
    OpenAIProvider,
    create_provider,
)
from auto_mcp.llm.base import BaseLLMProvider as BaseProvider
from auto_mcp.prompts.templates import (
    get_fallback_parameter_description,
    get_fallback_prompt_description,
    get_fallback_resource_description,
    get_fallback_tool_description,
)


@pytest.fixture
def sample_method_metadata() -> MethodMetadata:
    """Create sample method metadata for testing."""

    def sample_func(name: str, count: int = 10) -> str:
        """Process the given name and return a formatted string."""
        return f"{name}: {count}"

    return MethodMetadata(
        name="sample_func",
        qualified_name="sample_func",
        module_name="test_module",
        signature=inspect.signature(sample_func),
        docstring="Process the given name and return a formatted string.",
        type_hints={"name": str, "count": int},
        return_type=str,
        is_async=False,
        is_method=False,
        is_classmethod=False,
        is_staticmethod=False,
        source_code=(
            "def sample_func(name: str, count: int = 10) -> str:\n"
            "    return f'{name}: {count}'"
        ),
        decorators=[],
        parameters=[
            {
                "name": "name",
                "kind": "POSITIONAL_OR_KEYWORD",
                "has_default": False,
                "default": None,
                "type": str,
                "type_str": "str",
            },
            {
                "name": "count",
                "kind": "POSITIONAL_OR_KEYWORD",
                "has_default": True,
                "default": 10,
                "type": int,
                "type_str": "int",
            },
        ],
        mcp_metadata={},
    )


class TestLLMProviderProtocol:
    """Tests for LLMProvider protocol."""

    def test_ollama_implements_protocol(self) -> None:
        """Test that OllamaProvider implements LLMProvider protocol."""
        provider = OllamaProvider()
        assert isinstance(provider, LLMProvider)

    def test_openai_implements_protocol(self) -> None:
        """Test that OpenAIProvider implements LLMProvider protocol."""
        provider = OpenAIProvider()
        assert isinstance(provider, LLMProvider)

    def test_anthropic_implements_protocol(self) -> None:
        """Test that AnthropicProvider implements LLMProvider protocol."""
        provider = AnthropicProvider()
        assert isinstance(provider, LLMProvider)


class TestBaseLLMProvider:
    """Tests for BaseLLMProvider."""

    def test_model_name(self) -> None:
        """Test model_name property."""
        provider = BaseProvider("test-model")
        assert provider.model_name == "test-model"

    def test_build_tool_prompt(
        self, sample_method_metadata: MethodMetadata
    ) -> None:
        """Test tool prompt building."""
        provider = BaseProvider("test-model")
        prompt = provider._build_tool_prompt(sample_method_metadata, "Test context")

        assert "sample_func" in prompt
        assert "Process the given name" in prompt
        assert "Test context" in prompt

    def test_build_parameter_prompt(
        self, sample_method_metadata: MethodMetadata
    ) -> None:
        """Test parameter prompt building."""
        provider = BaseProvider("test-model")
        prompt = provider._build_parameter_prompt(sample_method_metadata)

        assert "sample_func" in prompt
        assert "name" in prompt
        assert "count" in prompt

    def test_parse_parameter_response(self) -> None:
        """Test parsing parameter descriptions from LLM response."""
        provider = BaseProvider("test-model")

        response = """
- name: The name to process
- count: Number of times to repeat
"""
        result = provider._parse_parameter_response(response)

        assert result["name"] == "The name to process"
        assert result["count"] == "Number of times to repeat"

    def test_parse_parameter_response_with_asterisks(self) -> None:
        """Test parsing with asterisk bullets."""
        provider = BaseProvider("test-model")

        response = """
* param1: First parameter description
* param2: Second parameter description
"""
        result = provider._parse_parameter_response(response)

        assert result["param1"] == "First parameter description"
        assert result["param2"] == "Second parameter description"

    def test_shutdown_is_noop(self) -> None:
        """Test that shutdown is a no-op by default."""
        provider = BaseProvider("test-model")
        # Should not raise
        provider.shutdown()


class TestOllamaProvider:
    """Tests for OllamaProvider."""

    def test_init_defaults(self) -> None:
        """Test default initialization."""
        provider = OllamaProvider()
        assert provider.model_name == "qwen2.5-coder:7b"
        assert provider.provider_name == "ollama"

    def test_init_custom_model(self) -> None:
        """Test initialization with custom model."""
        provider = OllamaProvider(model="codellama:7b")
        assert provider.model_name == "codellama:7b"

    def test_init_custom_host(self) -> None:
        """Test initialization with custom host."""
        provider = OllamaProvider(host="http://localhost:11435")
        assert provider._host == "http://localhost:11435"

    @pytest.mark.asyncio
    async def test_generate_tool_description_success(
        self, sample_method_metadata: MethodMetadata
    ) -> None:
        """Test successful tool description generation."""
        provider = OllamaProvider()

        mock_response = {"message": {"content": "Processes a name and returns formatted output."}}
        with patch.object(provider, "_get_client") as mock_get_client:
            mock_client = AsyncMock()
            mock_client.chat.return_value = mock_response
            mock_get_client.return_value = mock_client

            result = await provider.generate_tool_description(sample_method_metadata)

            assert result == "Processes a name and returns formatted output."
            mock_client.chat.assert_called_once()

    @pytest.mark.asyncio
    async def test_generate_tool_description_fallback(
        self, sample_method_metadata: MethodMetadata
    ) -> None:
        """Test fallback when LLM fails."""
        provider = OllamaProvider()

        with patch.object(provider, "_get_client") as mock_get_client:
            mock_client = AsyncMock()
            mock_client.chat.side_effect = Exception("Connection error")
            mock_get_client.return_value = mock_client

            result = await provider.generate_tool_description(sample_method_metadata)

            # Should return fallback based on docstring
            assert "Process the given name" in result

    @pytest.mark.asyncio
    async def test_generate_parameter_descriptions(
        self, sample_method_metadata: MethodMetadata
    ) -> None:
        """Test parameter description generation."""
        provider = OllamaProvider()

        mock_response = {
            "message": {"content": "- name: The name to format\n- count: Repeat count"}
        }
        with patch.object(provider, "_get_client") as mock_get_client:
            mock_client = AsyncMock()
            mock_client.chat.return_value = mock_response
            mock_get_client.return_value = mock_client

            result = await provider.generate_parameter_descriptions(sample_method_metadata)

            assert "name" in result
            assert "count" in result

    def test_shutdown(self) -> None:
        """Test shutdown clears client."""
        provider = OllamaProvider()
        provider._client = MagicMock()  # Simulate initialized client
        provider.shutdown()
        assert provider._client is None


class TestOpenAIProvider:
    """Tests for OpenAIProvider."""

    def test_init_defaults(self) -> None:
        """Test default initialization."""
        provider = OpenAIProvider()
        assert provider.model_name == "gpt-4o-mini"
        assert provider.provider_name == "openai"

    def test_init_custom_model(self) -> None:
        """Test initialization with custom model."""
        provider = OpenAIProvider(model="gpt-4o")
        assert provider.model_name == "gpt-4o"

    def test_init_with_api_key(self) -> None:
        """Test initialization with API key."""
        provider = OpenAIProvider(api_key="sk-test-key")
        assert provider._api_key == "sk-test-key"

    @pytest.mark.asyncio
    async def test_generate_tool_description_success(
        self, sample_method_metadata: MethodMetadata
    ) -> None:
        """Test successful tool description generation."""
        provider = OpenAIProvider()

        mock_message = MagicMock()
        mock_message.content = "Formats a name with the given count."
        mock_choice = MagicMock()
        mock_choice.message = mock_message
        mock_response = MagicMock()
        mock_response.choices = [mock_choice]

        with patch.object(provider, "_get_client") as mock_get_client:
            mock_client = AsyncMock()
            mock_client.chat.completions.create.return_value = mock_response
            mock_get_client.return_value = mock_client

            result = await provider.generate_tool_description(sample_method_metadata)

            assert result == "Formats a name with the given count."

    @pytest.mark.asyncio
    async def test_generate_tool_description_fallback(
        self, sample_method_metadata: MethodMetadata
    ) -> None:
        """Test fallback when API fails."""
        provider = OpenAIProvider()

        with patch.object(provider, "_get_client") as mock_get_client:
            mock_client = AsyncMock()
            mock_client.chat.completions.create.side_effect = Exception("API error")
            mock_get_client.return_value = mock_client

            result = await provider.generate_tool_description(sample_method_metadata)

            assert "Process the given name" in result


class TestAnthropicProvider:
    """Tests for AnthropicProvider."""

    def test_init_defaults(self) -> None:
        """Test default initialization."""
        provider = AnthropicProvider()
        assert provider.model_name == "claude-3-haiku-20240307"
        assert provider.provider_name == "anthropic"

    def test_init_custom_model(self) -> None:
        """Test initialization with custom model."""
        provider = AnthropicProvider(model="claude-3-sonnet-20240229")
        assert provider.model_name == "claude-3-sonnet-20240229"

    @pytest.mark.asyncio
    async def test_generate_tool_description_success(
        self, sample_method_metadata: MethodMetadata
    ) -> None:
        """Test successful tool description generation."""
        provider = AnthropicProvider()

        mock_block = MagicMock()
        mock_block.text = "Processes names with optional count."
        mock_response = MagicMock()
        mock_response.content = [mock_block]

        with patch.object(provider, "_get_client") as mock_get_client:
            mock_client = AsyncMock()
            mock_client.messages.create.return_value = mock_response
            mock_get_client.return_value = mock_client

            result = await provider.generate_tool_description(sample_method_metadata)

            assert result == "Processes names with optional count."


class TestCreateProvider:
    """Tests for create_provider factory function."""

    def test_create_ollama(self) -> None:
        """Test creating Ollama provider."""
        provider = create_provider("ollama")
        assert isinstance(provider, OllamaProvider)
        assert provider.model_name == "qwen2.5-coder:7b"

    def test_create_ollama_custom_model(self) -> None:
        """Test creating Ollama provider with custom model."""
        provider = create_provider("ollama", model="codellama:7b")
        assert provider.model_name == "codellama:7b"

    def test_create_openai(self) -> None:
        """Test creating OpenAI provider."""
        provider = create_provider("openai")
        assert isinstance(provider, OpenAIProvider)
        assert provider.model_name == "gpt-4o-mini"

    def test_create_openai_with_key(self) -> None:
        """Test creating OpenAI provider with API key."""
        provider = create_provider("openai", api_key="sk-test")
        assert isinstance(provider, OpenAIProvider)

    def test_create_anthropic(self) -> None:
        """Test creating Anthropic provider."""
        provider = create_provider("anthropic")
        assert isinstance(provider, AnthropicProvider)
        assert provider.model_name == "claude-3-haiku-20240307"

    def test_create_unknown_raises(self) -> None:
        """Test that unknown provider raises ValueError."""
        with pytest.raises(ValueError, match="Unknown provider"):
            create_provider("unknown")  # type: ignore[arg-type]


class TestFallbackDescriptions:
    """Tests for fallback description functions."""

    def test_tool_fallback_with_docstring(self) -> None:
        """Test tool fallback uses docstring first sentence."""
        result = get_fallback_tool_description(
            "my_func", "This is the first sentence. And more details."
        )
        assert result == "This is the first sentence."

    def test_tool_fallback_without_docstring(self) -> None:
        """Test tool fallback without docstring."""
        result = get_fallback_tool_description("my_func", None)
        assert "my_func" in result

    def test_resource_fallback_with_docstring(self) -> None:
        """Test resource fallback uses docstring."""
        result = get_fallback_resource_description("get_data", "Fetches data from source.")
        assert result == "Fetches data from source."

    def test_prompt_fallback(self) -> None:
        """Test prompt fallback."""
        result = get_fallback_prompt_description("greeting", "Generates a greeting.")
        assert result == "Generates a greeting."

    def test_parameter_fallback(self) -> None:
        """Test parameter fallback makes readable name."""
        result = get_fallback_parameter_description("user_name")
        assert "user name" in result



"""LLM provider integrations for auto-mcp."""

from typing import Literal

from auto_mcp.llm.anthropic import AnthropicProvider
from auto_mcp.llm.base import BaseLLMProvider, LLMProvider
from auto_mcp.llm.ollama import OllamaProvider
from auto_mcp.llm.openai import OpenAIProvider

__all__ = [
    "LLMProvider",
    "BaseLLMProvider",
    "OllamaProvider",
    "OpenAIProvider",
    "AnthropicProvider",
    "create_provider",
]


def create_provider(
    provider: Literal["ollama", "openai", "anthropic"],
    model: str | None = None,
    api_key: str | None = None,
    base_url: str | None = None,
) -> LLMProvider:
    """Create an LLM provider instance.

    Args:
        provider: The provider type to create
        model: Optional model name (uses provider default if not specified)
        api_key: Optional API key for cloud providers
        base_url: Optional custom base URL

    Returns:
        An LLM provider instance

    Raises:
        ValueError: If an unknown provider is specified
    """
    if provider == "ollama":
        return OllamaProvider(
            model=model or "qwen2.5-coder:7b",
            host=base_url,
        )
    elif provider == "openai":
        return OpenAIProvider(
            model=model or "gpt-4o-mini",
            api_key=api_key,
            base_url=base_url,
        )
    elif provider == "anthropic":
        return AnthropicProvider(
            model=model or "claude-3-haiku-20240307",
            api_key=api_key,
            base_url=base_url,
        )
    else:
        raise ValueError(f"Unknown provider: {provider}")



"""Tests for the high-level Python API."""

from __future__ import annotations

from pathlib import Path
from types import ModuleType

import pytest

from auto_mcp import AutoMCP, quick_server
from auto_mcp.api import AutoMCP as AutoMCPClass


def create_test_module() -> ModuleType:
    """Create a test module with sample functions."""
    module = ModuleType("test_api_module")
    module.__file__ = "/tmp/test_api_module.py"

    def add(a: int, b: int) -> int:
        """Add two numbers."""
        return a + b

    def greet(name: str) -> str:
        """Greet someone."""
        return f"Hello, {name}!"

    async def async_func(data: str) -> str:
        """Async function."""
        return f"processed: {data}"

    # Set __module__ for proper inspection
    add.__module__ = "test_api_module"
    greet.__module__ = "test_api_module"
    async_func.__module__ = "test_api_module"

    module.add = add
    module.greet = greet
    module.async_func = async_func

    return module


class TestAutoMCPInit:
    """Tests for AutoMCP initialization."""

    def test_default_init(self) -> None:
        """Test default initialization."""
        auto = AutoMCP(use_llm=False)

        assert auto.llm_provider is None
        assert auto.config is not None
        assert auto.config.server_name == "auto-mcp-server"

    def test_init_with_server_name(self) -> None:
        """Test initialization with custom server name."""
        auto = AutoMCP(use_llm=False, server_name="my-server")

        assert auto.config.server_name == "my-server"

    def test_init_with_llm_provider(self) -> None:
        """Test initialization with LLM provider."""
        auto = AutoMCP(llm_provider="ollama", llm_model="test-model")

        assert auto.llm_provider is not None
        assert auto.llm_provider.model_name == "test-model"

    def test_init_without_llm(self) -> None:
        """Test initialization without LLM."""
        auto = AutoMCP(use_llm=False)

        assert auto.llm_provider is None
        assert auto.config.use_llm is False

    def test_init_without_cache(self) -> None:
        """Test initialization without cache."""
        auto = AutoMCP(use_llm=False, use_cache=False)

        assert auto.config.use_cache is False

    def test_init_include_private(self) -> None:
        """Test initialization with include_private."""
        auto = AutoMCP(use_llm=False, include_private=True)

        assert auto.config.include_private is True

    def test_init_no_resources(self) -> None:
        """Test initialization without resources."""
        auto = AutoMCP(use_llm=False, generate_resources=False)

        assert auto.config.generate_resources is False

    def test_init_no_prompts(self) -> None:
        """Test initialization without prompts."""
        auto = AutoMCP(use_llm=False, generate_prompts=False)

        assert auto.config.generate_prompts is False


class TestAutoMCPCreateServer:
    """Tests for create_server method."""

    def test_create_server(self) -> None:
        """Test creating a server."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=False)

        server = auto.create_server([module])

        assert server is not None
        assert server.name == "auto-mcp-server"

    def test_create_server_custom_name(self) -> None:
        """Test creating a server with custom name."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=False)

        server = auto.create_server([module], name="custom-server")

        assert server.name == "custom-server"

    def test_create_server_restores_config(self) -> None:
        """Test that config is restored after create_server with custom name."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=False, server_name="original")

        auto.create_server([module], name="temporary")

        assert auto.config.server_name == "original"

    def test_create_server_multiple_modules(self) -> None:
        """Test creating a server from multiple modules."""
        module1 = create_test_module()
        module2 = ModuleType("test_module_2")
        module2.__file__ = "/tmp/test_module_2.py"

        def subtract(a: int, b: int) -> int:
            """Subtract two numbers."""
            return a - b

        subtract.__module__ = "test_module_2"
        module2.subtract = subtract

        auto = AutoMCP(use_llm=False, use_cache=False)
        server = auto.create_server([module1, module2])

        assert server is not None


class TestAutoMCPGenerateFile:
    """Tests for generate_file method."""

    def test_generate_file(self, tmp_path: Path) -> None:
        """Test generating a standalone file."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=False)
        output = tmp_path / "server.py"

        result = auto.generate_file([module], output)

        assert result == output
        assert output.exists()

        content = output.read_text()
        assert "FastMCP" in content
        assert "@mcp.tool" in content

    def test_generate_file_custom_name(self, tmp_path: Path) -> None:
        """Test generating a file with custom server name."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=False)
        output = tmp_path / "server.py"

        auto.generate_file([module], output, name="my-server")

        content = output.read_text()
        assert "my-server" in content

    def test_generate_file_string_path(self, tmp_path: Path) -> None:
        """Test generating a file with string path."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=False)
        output = str(tmp_path / "server.py")

        result = auto.generate_file([module], output)

        assert result.exists()


class TestAutoMCPGeneratePackage:
    """Tests for generate_package method."""

    def test_generate_package(self, tmp_path: Path) -> None:
        """Test generating a package."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=False)

        result = auto.generate_package([module], tmp_path, "my-package")

        assert result == tmp_path / "my-package"
        assert result.exists()
        assert (result / "pyproject.toml").exists()
        assert (result / "src" / "my_package" / "server.py").exists()

    def test_generate_package_string_path(self, tmp_path: Path) -> None:
        """Test generating a package with string path."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=False)

        result = auto.generate_package([module], str(tmp_path), "my-pkg")

        assert result.exists()


class TestAutoMCPAnalyze:
    """Tests for analyze methods."""

    @pytest.mark.asyncio
    async def test_analyze(self) -> None:
        """Test async analyze method."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=False)

        tools, resources, prompts = await auto.analyze([module])

        assert len(tools) >= 2
        tool_names = [t.name for t in tools]
        assert "add" in tool_names
        assert "greet" in tool_names

    def test_analyze_sync(self) -> None:
        """Test sync analyze method."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=False)

        tools, resources, prompts = auto.analyze_sync([module])

        assert len(tools) >= 2


class TestAutoMCPCache:
    """Tests for cache methods."""

    def test_save_cache(self, tmp_path: Path) -> None:
        """Test saving cache."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=True, cache_dir=tmp_path)

        # Generate something to populate cache
        auto.create_server([module])

        # Should not raise
        auto.save_cache([module])

    def test_save_cache_all(self, tmp_path: Path) -> None:
        """Test saving all cache."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=True, cache_dir=tmp_path)

        auto.create_server([module])
        auto.save_cache()  # Save all

    def test_clear_cache(self, tmp_path: Path) -> None:
        """Test clearing cache."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=True, cache_dir=tmp_path)

        auto.create_server([module])
        count = auto.clear_cache([module])

        # Count depends on what was cached
        assert count >= 0

    def test_clear_cache_all(self, tmp_path: Path) -> None:
        """Test clearing all cache."""
        module = create_test_module()
        auto = AutoMCP(use_llm=False, use_cache=True, cache_dir=tmp_path)

        auto.create_server([module])
        count = auto.clear_cache()

        assert count >= 0


class TestAutoMCPContextManager:
    """Tests for context manager functionality."""

    def test_context_manager(self, tmp_path: Path) -> None:
        """Test using AutoMCP as context manager."""
        module = create_test_module()

        with AutoMCP(use_llm=False, use_cache=True, cache_dir=tmp_path) as auto:
            server = auto.create_server([module])
            assert server is not None

        # After exit, should have saved cache (no error)

    def test_shutdown(self) -> None:
        """Test explicit shutdown."""
        auto = AutoMCP(use_llm=False, use_cache=False)

        # Should not raise
        auto.shutdown()


class TestQuickServer:
    """Tests for quick_server convenience function."""

    def test_quick_server(self) -> None:
        """Test quick_server function."""
        module = create_test_module()

        server = quick_server(module)

        assert server is not None
        assert server.name == "auto-mcp-server"

    def test_quick_server_custom_name(self) -> None:
        """Test quick_server with custom name."""
        module = create_test_module()

        server = quick_server(module, name="quick-server")

        assert server.name == "quick-server"

    def test_quick_server_multiple_modules(self) -> None:
        """Test quick_server with multiple modules."""
        module1 = create_test_module()
        module2 = ModuleType("another_module")
        module2.__file__ = "/tmp/another_module.py"

        def func() -> str:
            """A function."""
            return "result"

        func.__module__ = "another_module"
        module2.func = func

        server = quick_server(module1, module2)

        assert server is not None


class TestAutoMCPImports:
    """Tests for package imports."""

    def test_import_from_package(self) -> None:
        """Test importing AutoMCP from package."""
        from auto_mcp import AutoMCP as ImportedAutoMCP

        assert ImportedAutoMCP is AutoMCPClass

    def test_import_quick_server(self) -> None:
        """Test importing quick_server from package."""
        from auto_mcp import quick_server as imported_quick_server

        assert imported_quick_server is quick_server

    def test_all_exports(self) -> None:
        """Test that all expected exports are available."""
        import auto_mcp

        assert hasattr(auto_mcp, "AutoMCP")
        assert hasattr(auto_mcp, "quick_server")
        assert hasattr(auto_mcp, "MCPGenerator")
        assert hasattr(auto_mcp, "GeneratorConfig")
        assert hasattr(auto_mcp, "Settings")
        assert hasattr(auto_mcp, "get_settings")
        assert hasattr(auto_mcp, "mcp_tool")
        assert hasattr(auto_mcp, "mcp_exclude")
        assert hasattr(auto_mcp, "mcp_resource")
        assert hasattr(auto_mcp, "mcp_prompt")


class TestAutoMCPFromPackage:
    """Tests for package-based methods."""

    def test_create_server_from_package(self) -> None:
        """Test creating server from package."""
        auto = AutoMCP(use_llm=False, use_cache=False)

        # Use collections as test package (stdlib, always available)
        server = auto.create_server_from_package("collections")

        assert server is not None

    def test_create_server_from_package_with_name(self) -> None:
        """Test creating server from package with custom name."""
        auto = AutoMCP(use_llm=False, use_cache=False, server_name="original")

        server = auto.create_server_from_package("collections", name="custom-package-server")

        assert server.name == "custom-package-server"
        # Original config should be restored
        assert auto.config.server_name == "original"

    def test_generate_file_from_package(self, tmp_path: Path) -> None:
        """Test generating file from package."""
        auto = AutoMCP(use_llm=False, use_cache=False)
        output = tmp_path / "package_server.py"

        result = auto.generate_file_from_package("collections", output)

        assert result.exists()
        content = result.read_text()
        assert "from mcp.server.fastmcp import FastMCP" in content

    def test_generate_file_from_package_with_name(self, tmp_path: Path) -> None:
        """Test generating file from package with custom name."""
        auto = AutoMCP(use_llm=False, use_cache=False, server_name="original")
        output = tmp_path / "custom_server.py"

        result = auto.generate_file_from_package(
            "collections", output, name="custom-name"
        )

        assert result.exists()
        content = result.read_text()
        assert 'mcp = FastMCP(name="custom-name")' in content
        # Original config should be restored
        assert auto.config.server_name == "original"

    def test_analyze_package(self) -> None:
        """Test analyzing a package."""
        auto = AutoMCP(use_llm=False, use_cache=False)

        metadata = auto.analyze_package("collections")

        assert metadata is not None
        assert metadata.name == "collections"
        assert metadata.module_count >= 1

    @pytest.mark.asyncio
    async def test_analyze_package_async(self) -> None:
        """Test async analyze package."""
        auto = AutoMCP(use_llm=False, use_cache=False)

        tools, resources, prompts = await auto.analyze_package_async("collections")

        # Should have at least some tools
        assert isinstance(tools, list)
        assert isinstance(resources, list)
        assert isinstance(prompts, list)

    def test_analyze_package_sync(self) -> None:
        """Test sync analyze package."""
        auto = AutoMCP(use_llm=False, use_cache=False)

        tools, resources, prompts = auto.analyze_package_sync("collections")

        # Should have at least some tools
        assert isinstance(tools, list)
        assert isinstance(resources, list)
        assert isinstance(prompts, list)


class TestAutoMCPWithLLM:
    """Tests for AutoMCP with LLM integration."""

    def test_with_mock_llm(self) -> None:
        """Test AutoMCP with mocked LLM."""
        # Create with Ollama but we won't actually call it
        auto = AutoMCP(
            llm_provider="ollama",
            llm_model="test-model",
            use_cache=False,
        )

        assert auto.llm_provider is not None
        assert auto.llm_provider.model_name == "test-model"

    def test_llm_provider_shutdown(self) -> None:
        """Test that LLM provider is shut down properly."""
        auto = AutoMCP(
            llm_provider="ollama",
            llm_model="test-model",
        )

        # Shutdown should call provider's shutdown
        auto.shutdown()

        # No error means success



#!/usr/bin/env python3
"""Test script for pandas MCP server.

This script demonstrates:
1. Creating DataFrames through MCP tools
2. Aggregation calculations using pandas via MCP
3. DataFrame serialization/deserialization verification

Run the pandas server first:
    python pandas_server.py

Then run this test in another terminal:
    python test_pandas_mcp.py
"""

from __future__ import annotations

import asyncio
import json
import sys
from pathlib import Path

# Add src to path for development
src_path = Path(__file__).parent.parent.parent / "src"
sys.path.insert(0, str(src_path))

import pandas as pd

from auto_mcp import AutoMCP
from auto_mcp.core.generator import GeneratorConfig
from auto_mcp.types import get_default_registry, register_all_adapters, create_pandas_dataframe_adapter


def test_direct_function_calls():
    """Test pandas functions directly to verify they work."""
    print("\n=== Direct Pandas Function Tests ===\n")

    # Test 1: Create DataFrame using pd.DataFrame (a class, not exposed as tool)
    print("1. Creating DataFrame directly...")
    df = pd.DataFrame({
        "product": ["Laptop", "Phone", "Tablet", "Laptop", "Phone"],
        "quantity": [5, 12, 8, 3, 15],
        "price": [1200.0, 800.0, 500.0, 1200.0, 800.0],
        "region": ["North", "South", "East", "West", "North"],
    })
    df["revenue"] = df["quantity"] * df["price"]
    print(f"   Created DataFrame with shape: {df.shape}")
    print(df.to_string(index=False))

    # Test 2: Use concat (an MCP tool)
    print("\n2. Testing pd.concat (exposed as MCP tool)...")
    df1 = pd.DataFrame({"A": [1, 2], "B": [3, 4]})
    df2 = pd.DataFrame({"A": [5, 6], "B": [7, 8]})
    result = pd.concat([df1, df2], ignore_index=True)
    print(f"   Concatenated DataFrame:")
    print(result.to_string(index=False))

    # Test 3: Use merge (an MCP tool)
    print("\n3. Testing pd.merge (exposed as MCP tool)...")
    left = pd.DataFrame({"key": ["A", "B", "C"], "value": [1, 2, 3]})
    right = pd.DataFrame({"key": ["B", "C", "D"], "other": [4, 5, 6]})
    merged = pd.merge(left, right, on="key", how="inner")
    print(f"   Merged DataFrame:")
    print(merged.to_string(index=False))

    # Test 4: Use pivot_table (an MCP tool)
    print("\n4. Testing pd.pivot_table (exposed as MCP tool)...")
    pivot = pd.pivot_table(
        df,
        values="revenue",
        index="region",
        columns="product",
        aggfunc="sum",
        fill_value=0,
    )
    print(f"   Pivot table (Revenue by Region and Product):")
    print(pivot.to_string())

    # Test 5: Use melt (an MCP tool)
    print("\n5. Testing pd.melt (exposed as MCP tool)...")
    wide_df = pd.DataFrame({
        "id": [1, 2],
        "name": ["Alice", "Bob"],
        "math_score": [90, 85],
        "english_score": [88, 92],
    })
    melted = pd.melt(
        wide_df,
        id_vars=["id", "name"],
        value_vars=["math_score", "english_score"],
        var_name="subject",
        value_name="score",
    )
    print(f"   Melted DataFrame:")
    print(melted.to_string(index=False))

    print("\n✓ All direct function tests passed!")
    return df


def test_type_serialization():
    """Test DataFrame serialization through the type system."""
    print("\n=== Type Serialization Tests ===\n")

    # Register adapters including pandas
    registry = get_default_registry()
    register_all_adapters(registry)

    # Create test DataFrame
    df = pd.DataFrame({
        "name": ["Alice", "Bob", "Charlie"],
        "age": [30, 25, 35],
        "salary": [50000.0, 45000.0, 60000.0],
    })

    # Check if DataFrame adapter is registered
    adapter = registry.get_adapter(pd.DataFrame)
    if adapter:
        print("1. DataFrame adapter registered ✓")

        # Test serialization
        serialized = adapter.serialize(df)
        print(f"\n2. Serialized DataFrame:")
        print(f"   Columns: {serialized['columns']}")
        print(f"   Shape: {serialized['shape']}")
        print(f"   Dtypes: {serialized['dtypes']}")
        print(f"   Data (first 2 rows): {serialized['data'][:2]}")

        # Test deserialization
        deserialized = adapter.deserialize(serialized)
        print(f"\n3. Deserialized DataFrame:")
        print(deserialized.to_string(index=False))

        # Verify round-trip
        assert list(deserialized.columns) == list(df.columns), "Columns mismatch"
        assert len(deserialized) == len(df), "Row count mismatch"
        print("\n✓ Round-trip serialization verified!")
    else:
        print("DataFrame adapter not available")


def test_mcp_server_creation():
    """Test creating an MCP server with pandas functions."""
    print("\n=== MCP Server Creation Test ===\n")

    # Note: Creating in-memory servers from pandas is challenging because
    # FastMCP's func_metadata tries to resolve type annotations, and pandas
    # has internal types that can't be resolved at runtime.
    #
    # The generated standalone server file (pandas_server.py) works because
    # it uses *args, **kwargs wrappers that bypass type annotation resolution.

    print("1. Testing generated server file exists...")
    server_path = Path(__file__).parent / "pandas_server.py"
    if server_path.exists():
        print(f"   ✓ Server file exists: {server_path.name}")

        # Read and verify the server has tools
        content = server_path.read_text()
        tool_count = content.count("@mcp.tool(")
        print(f"   ✓ Server has {tool_count} registered tools")
    else:
        print("   ! Server file not found. Run:")
        print("     auto-mcp package generate pandas -o pandas_server.py --no-llm --include-reexports")

    # Test with a simpler package (json) to verify API works
    print("\n2. Testing AutoMCP API with 'json' package...")
    try:
        auto = AutoMCP(
            use_llm=False,
            server_name="json-test-server",
            include_reexports=True,
            max_depth=0,
        )
        server = auto.create_server_from_package("json")
        print(f"   ✓ Server created: {server.name}")
        print("   Server is ready to accept MCP connections")
    except Exception as e:
        print(f"   ! Server creation failed: {e}")

    print("\n✓ MCP server tests completed!")
    return None


def test_aggregation_example():
    """Demonstrate aggregation calculations that would work through MCP."""
    print("\n=== Aggregation Example ===\n")

    # Create sample sales data
    sales_data = pd.DataFrame({
        "date": pd.date_range("2024-01-01", periods=10, freq="D"),
        "product": ["Laptop", "Phone", "Tablet"] * 3 + ["Laptop"],
        "quantity": [5, 12, 8, 3, 15, 7, 4, 20, 6, 9],
        "price": [1200.0, 800.0, 500.0, 1200.0, 800.0, 1200.0, 500.0, 800.0, 1200.0, 500.0],
        "region": ["North", "South", "East", "West", "North", "South", "East", "West", "North", "South"],
    })
    sales_data["revenue"] = sales_data["quantity"] * sales_data["price"]

    print("Sales Data:")
    print(sales_data.to_string(index=False))

    # Aggregation 1: Total revenue by product using pivot_table
    print("\n1. Revenue by Product (using pd.pivot_table):")
    product_revenue = pd.pivot_table(
        sales_data,
        values="revenue",
        index="product",
        aggfunc="sum",
    )
    print(product_revenue.to_string())

    # Aggregation 2: Revenue by region
    print("\n2. Revenue by Region (using pd.pivot_table):")
    region_revenue = pd.pivot_table(
        sales_data,
        values="revenue",
        index="region",
        aggfunc=["sum", "mean", "count"],
    )
    print(region_revenue.to_string())

    # Aggregation 3: Cross-tabulation
    print("\n3. Quantity Cross-tabulation (using pd.crosstab):")
    cross = pd.crosstab(
        sales_data["region"],
        sales_data["product"],
        values=sales_data["quantity"],
        aggfunc="sum",
        margins=True,
    )
    print(cross.to_string())

    # Summary statistics
    print("\n4. Summary Statistics:")
    print(f"   Total Revenue: ${sales_data['revenue'].sum():,.2f}")
    print(f"   Average Revenue per Sale: ${sales_data['revenue'].mean():,.2f}")
    print(f"   Total Quantity Sold: {sales_data['quantity'].sum()}")
    print(f"   Number of Transactions: {len(sales_data)}")

    print("\n✓ Aggregation examples completed!")


def main():
    """Run all tests."""
    print("=" * 60)
    print("Pandas MCP Server - Test Suite")
    print("=" * 60)

    # Run tests
    test_direct_function_calls()
    test_type_serialization()
    test_aggregation_example()
    test_mcp_server_creation()

    print("\n" + "=" * 60)
    print("All tests completed successfully!")
    print("=" * 60)


if __name__ == "__main__":
    main()



"""Compression support for type adapters.

This module provides compression wrappers for handling large serialized data.
"""

from __future__ import annotations

import base64
import gzip
import json
import logging
import zlib
from dataclasses import dataclass
from enum import Enum
from typing import Any, Generic, TypeVar

from auto_mcp.types.base import JsonValue, TypeAdapter

logger = logging.getLogger(__name__)

T = TypeVar("T")


class CompressionAlgorithm(Enum):
    """Supported compression algorithms."""

    GZIP = "gzip"
    ZLIB = "zlib"
    LZ4 = "lz4"  # Optional, requires lz4 package
    NONE = "none"


@dataclass
class CompressionConfig:
    """Configuration for data compression.

    Attributes:
        enabled: Whether compression is enabled
        algorithm: Compression algorithm to use
        threshold: Minimum size in bytes to trigger compression
        level: Compression level (1-9, higher = more compression)
    """

    enabled: bool = True
    algorithm: CompressionAlgorithm = CompressionAlgorithm.GZIP
    threshold: int = 1024  # 1KB default threshold
    level: int = 6  # Default compression level


@dataclass
class CompressedData:
    """Container for compressed data with metadata.

    Attributes:
        compressed: Whether the data is compressed
        algorithm: Algorithm used for compression
        original_size: Size before compression
        compressed_size: Size after compression
        data: The (possibly compressed) data as base64 string
    """

    compressed: bool
    algorithm: str
    original_size: int
    compressed_size: int
    data: str

    def to_dict(self) -> dict[str, Any]:
        """Convert to JSON-serializable dict."""
        return {
            "compressed": self.compressed,
            "algorithm": self.algorithm,
            "original_size": self.original_size,
            "compressed_size": self.compressed_size,
            "compression_ratio": round(self.compressed_size / self.original_size, 3)
            if self.original_size > 0
            else 1.0,
            "data": self.data,
        }

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> CompressedData:
        """Create from dict."""
        return cls(
            compressed=data["compressed"],
            algorithm=data["algorithm"],
            original_size=data["original_size"],
            compressed_size=data["compressed_size"],
            data=data["data"],
        )


def compress_bytes(
    data: bytes,
    algorithm: CompressionAlgorithm = CompressionAlgorithm.GZIP,
    level: int = 6,
) -> bytes:
    """Compress bytes using the specified algorithm.

    Args:
        data: Data to compress
        algorithm: Compression algorithm
        level: Compression level (1-9)

    Returns:
        Compressed bytes
    """
    if algorithm == CompressionAlgorithm.GZIP:
        return gzip.compress(data, compresslevel=level)

    if algorithm == CompressionAlgorithm.ZLIB:
        return zlib.compress(data, level=level)

    if algorithm == CompressionAlgorithm.LZ4:
        try:
            import lz4.frame

            return lz4.frame.compress(data, compression_level=level)
        except ImportError:
            logger.warning("lz4 not installed, falling back to gzip")
            return gzip.compress(data, compresslevel=level)

    # No compression
    return data


def decompress_bytes(
    data: bytes,
    algorithm: CompressionAlgorithm = CompressionAlgorithm.GZIP,
) -> bytes:
    """Decompress bytes using the specified algorithm.

    Args:
        data: Compressed data
        algorithm: Compression algorithm used

    Returns:
        Decompressed bytes
    """
    if algorithm == CompressionAlgorithm.GZIP:
        return gzip.decompress(data)

    if algorithm == CompressionAlgorithm.ZLIB:
        return zlib.decompress(data)

    if algorithm == CompressionAlgorithm.LZ4:
        try:
            import lz4.frame

            return lz4.frame.decompress(data)
        except ImportError as e:
            raise ImportError(
                "lz4 package required to decompress lz4 data"
            ) from e

    # No compression
    return data


class CompressedAdapter(TypeAdapter[T], Generic[T]):
    """Wrapper adapter that adds compression to any TypeAdapter.

    This adapter wraps another adapter and automatically compresses
    the serialized data if it exceeds the configured threshold.

    Example:
        >>> base_adapter = DataFrameAdapter()
        >>> compressed = CompressedAdapter(
        ...     base_adapter,
        ...     config=CompressionConfig(threshold=10240)  # 10KB
        ... )
        >>> # Large DataFrames will be automatically compressed
        >>> serialized = compressed.serialize(large_df)
    """

    def __init__(
        self,
        base_adapter: TypeAdapter[T],
        config: CompressionConfig | None = None,
    ) -> None:
        """Initialize compressed adapter.

        Args:
            base_adapter: The underlying adapter to wrap
            config: Compression configuration
        """
        self.base_adapter = base_adapter
        self.config = config or CompressionConfig()
        self.target_type = base_adapter.target_type

    def serialize(self, obj: T) -> JsonValue:
        """Serialize and optionally compress the object.

        Args:
            obj: Object to serialize

        Returns:
            JSON-compatible value (dict with compression metadata if compressed)
        """
        # First serialize with base adapter
        serialized = self.base_adapter.serialize(obj)

        # Convert to JSON bytes for size check
        json_bytes = json.dumps(serialized).encode("utf-8")
        original_size = len(json_bytes)

        # Check if compression should be applied
        if not self.config.enabled or original_size < self.config.threshold:
            # Return uncompressed with metadata
            return CompressedData(
                compressed=False,
                algorithm="none",
                original_size=original_size,
                compressed_size=original_size,
                data=base64.b64encode(json_bytes).decode("ascii"),
            ).to_dict()

        # Compress the data
        compressed_bytes = compress_bytes(
            json_bytes,
            algorithm=self.config.algorithm,
            level=self.config.level,
        )
        compressed_size = len(compressed_bytes)

        # Only use compression if it actually saves space
        if compressed_size >= original_size:
            return CompressedData(
                compressed=False,
                algorithm="none",
                original_size=original_size,
                compressed_size=original_size,
                data=base64.b64encode(json_bytes).decode("ascii"),
            ).to_dict()

        logger.debug(
            f"Compressed {original_size} bytes to {compressed_size} bytes "
            f"({100 * compressed_size / original_size:.1f}%)"
        )

        return CompressedData(
            compressed=True,
            algorithm=self.config.algorithm.value,
            original_size=original_size,
            compressed_size=compressed_size,
            data=base64.b64encode(compressed_bytes).decode("ascii"),
        ).to_dict()

    def deserialize(self, data: JsonValue) -> T:
        """Decompress and deserialize the data.

        Args:
            data: JSON-compatible value (possibly compressed)

        Returns:
            The deserialized object
        """
        if not isinstance(data, dict) or "compressed" not in data:
            # Not wrapped in compression metadata, pass through
            return self.base_adapter.deserialize(data)

        compressed_data = CompressedData.from_dict(data)
        raw_bytes = base64.b64decode(compressed_data.data)

        if compressed_data.compressed:
            # Decompress
            algorithm = CompressionAlgorithm(compressed_data.algorithm)
            raw_bytes = decompress_bytes(raw_bytes, algorithm)

        # Parse JSON and deserialize
        json_data = json.loads(raw_bytes.decode("utf-8"))
        return self.base_adapter.deserialize(json_data)

    def json_schema(self) -> dict[str, Any]:
        """Return JSON Schema for compressed data."""
        return {
            "type": "object",
            "properties": {
                "compressed": {
                    "type": "boolean",
                    "description": "Whether the data is compressed",
                },
                "algorithm": {
                    "type": "string",
                    "enum": ["gzip", "zlib", "lz4", "none"],
                    "description": "Compression algorithm used",
                },
                "original_size": {
                    "type": "integer",
                    "description": "Original data size in bytes",
                },
                "compressed_size": {
                    "type": "integer",
                    "description": "Compressed data size in bytes",
                },
                "compression_ratio": {
                    "type": "number",
                    "description": "Ratio of compressed to original size",
                },
                "data": {
                    "type": "string",
                    "contentEncoding": "base64",
                    "description": "Base64-encoded (possibly compressed) data",
                },
            },
            "required": ["compressed", "algorithm", "data"],
            "description": f"Compressed {self.base_adapter.target_type.__name__}",
        }

    def can_serialize(self, obj: T) -> bool:
        """Check if the base adapter can serialize this object."""
        return self.base_adapter.can_serialize(obj)


def with_compression(
    adapter: TypeAdapter[T],
    threshold: int = 1024,
    algorithm: CompressionAlgorithm = CompressionAlgorithm.GZIP,
    level: int = 6,
) -> CompressedAdapter[T]:
    """Convenience function to wrap an adapter with compression.

    Args:
        adapter: The adapter to wrap
        threshold: Minimum size to trigger compression (bytes)
        algorithm: Compression algorithm
        level: Compression level

    Returns:
        CompressedAdapter wrapping the original adapter
    """
    config = CompressionConfig(
        enabled=True,
        algorithm=algorithm,
        threshold=threshold,
        level=level,
    )
    return CompressedAdapter(adapter, config)


class AutoCompressRegistry:
    """Registry extension that automatically applies compression to large types.

    This is a helper that can wrap adapters in a TypeRegistry with compression
    based on configuration.
    """

    def __init__(
        self,
        config: CompressionConfig | None = None,
        large_type_threshold: int = 10240,  # 10KB
    ) -> None:
        """Initialize auto-compress registry.

        Args:
            config: Compression configuration
            large_type_threshold: Types that commonly produce large output
        """
        self.config = config or CompressionConfig()
        self.large_type_threshold = large_type_threshold

        # Types that typically produce large serialized output
        self._large_types: set[type] = set()

    def register_large_type(self, type_: type) -> None:
        """Register a type as typically producing large output.

        Args:
            type_: The type to register
        """
        self._large_types.add(type_)

    def wrap_adapter(self, adapter: TypeAdapter[T]) -> TypeAdapter[T]:
        """Wrap an adapter with compression if appropriate.

        Args:
            adapter: The adapter to potentially wrap

        Returns:
            The adapter, possibly wrapped with compression
        """
        if not self.config.enabled:
            return adapter

        # Always wrap types registered as large
        if adapter.target_type in self._large_types:
            return CompressedAdapter(adapter, self.config)

        # Otherwise return as-is (threshold will be checked at serialize time)
        return adapter

    def wrap_if_large(
        self,
        adapter: TypeAdapter[T],
        threshold: int | None = None,
    ) -> CompressedAdapter[T]:
        """Explicitly wrap an adapter with compression.

        Args:
            adapter: The adapter to wrap
            threshold: Override the default threshold

        Returns:
            CompressedAdapter wrapping the original
        """
        config = CompressionConfig(
            enabled=True,
            algorithm=self.config.algorithm,
            threshold=threshold or self.large_type_threshold,
            level=self.config.level,
        )
        return CompressedAdapter(adapter, config)



"""Type registry for managing type adapters and strategies."""

from __future__ import annotations

import logging
from typing import Any, TypeVar, cast

from auto_mcp.types.base import (
    FunctionAdapter,
    JsonValue,
    ObjectStoreConfig,
    TypeAdapter,
    TypeInfo,
    TypeStrategy,
    is_json_compatible,
)

logger = logging.getLogger(__name__)

T = TypeVar("T")


class TypeRegistry:
    """Central registry for type handling strategies.

    The registry manages type adapters and object store configurations,
    providing a unified interface for type transformations.

    Example:
        >>> registry = TypeRegistry()
        >>>
        >>> # Register an adapter
        >>> registry.register_adapter(DateTimeAdapter())
        >>>
        >>> # Or use the quick registration
        >>> registry.register(
        ...     datetime,
        ...     serialize=lambda dt: dt.isoformat(),
        ...     deserialize=lambda s: datetime.fromisoformat(s),
        ...     schema={"type": "string", "format": "date-time"},
        ... )
        >>>
        >>> # Register a stored type
        >>> registry.register_stored_type(
        ...     requests.Session,
        ...     ttl=3600,
        ...     auto_create_tool=True,
        ... )
        >>>
        >>> # Get type info
        >>> info = registry.get_type_info(datetime)
        >>> print(info.strategy)  # TypeStrategy.ADAPTER
    """

    def __init__(self) -> None:
        """Initialize an empty type registry."""
        self._adapters: dict[type, TypeAdapter[Any]] = {}
        self._store_configs: dict[type, ObjectStoreConfig] = {}
        self._type_cache: dict[type, TypeInfo] = {}

    def register_adapter(self, adapter: TypeAdapter[T]) -> None:
        """Register a type adapter.

        Args:
            adapter: The adapter instance to register

        Raises:
            ValueError: If an adapter for this type is already registered
        """
        target_type = adapter.target_type

        if target_type in self._adapters:
            logger.warning(
                f"Overwriting existing adapter for {target_type.__name__}"
            )

        self._adapters[target_type] = adapter
        self._invalidate_cache(target_type)

        logger.debug(f"Registered adapter for {target_type.__name__}")

    def register(
        self,
        target_type: type[T],
        *,
        serialize: Any,  # Callable[[T], JsonValue]
        deserialize: Any,  # Callable[[JsonValue], T]
        schema: dict[str, Any] | None = None,
        can_serialize: Any | None = None,  # Callable[[T], bool]
    ) -> None:
        """Quick registration of a type adapter using functions.

        Args:
            target_type: The type to register an adapter for
            serialize: Function to convert object to JSON
            deserialize: Function to convert JSON to object
            schema: JSON Schema for the serialized form
            can_serialize: Optional function to check if object can be serialized
        """
        adapter = FunctionAdapter(
            target_type=target_type,
            serializer=serialize,
            deserializer=deserialize,
            schema=schema,
            can_serialize_fn=can_serialize,
        )
        self.register_adapter(adapter)

    def register_stored_type(
        self,
        target_type: type,
        *,
        ttl: int = 3600,
        max_instances: int = 100,
        singleton: bool = False,
        auto_create_tool: bool = True,
        auto_cleanup_tool: bool = True,
        factory: Any | None = None,
    ) -> None:
        """Register a type to be stored server-side with handles.

        Args:
            target_type: The type to register
            ttl: Time-to-live in seconds (0 for no expiration)
            max_instances: Maximum number of instances to store
            singleton: Only allow one instance of this type
            auto_create_tool: Auto-generate a creation tool
            auto_cleanup_tool: Auto-generate a cleanup tool
            factory: Optional factory function to create instances
        """
        config = ObjectStoreConfig(
            ttl=ttl,
            max_instances=max_instances,
            singleton=singleton,
            auto_create_tool=auto_create_tool,
            auto_cleanup_tool=auto_cleanup_tool,
            factory=factory,
        )

        if target_type in self._store_configs:
            logger.warning(
                f"Overwriting existing store config for {target_type.__name__}"
            )

        self._store_configs[target_type] = config
        self._invalidate_cache(target_type)

        logger.debug(f"Registered stored type {target_type.__name__}")

    def get_adapter(self, type_: type) -> TypeAdapter[Any] | None:
        """Get the adapter for a type.

        Args:
            type_: The type to get an adapter for

        Returns:
            The adapter instance or None if not registered
        """
        # Direct lookup
        if type_ in self._adapters:
            return self._adapters[type_]

        # Check for parent class adapters
        for registered_type, adapter in self._adapters.items():
            if isinstance(type_, type) and issubclass(type_, registered_type):
                return adapter

        return None

    def get_store_config(self, type_: type) -> ObjectStoreConfig | None:
        """Get the object store config for a type.

        Args:
            type_: The type to get config for

        Returns:
            The config or None if not registered
        """
        # Direct lookup
        if type_ in self._store_configs:
            return self._store_configs[type_]

        # Check for parent class configs
        for registered_type, config in self._store_configs.items():
            if isinstance(type_, type) and issubclass(type_, registered_type):
                return config

        return None

    def get_type_info(self, type_: type) -> TypeInfo:
        """Get complete type information including strategy.

        Args:
            type_: The type to get info for

        Returns:
            TypeInfo with strategy and handler details
        """
        # Check cache
        if type_ in self._type_cache:
            return self._type_cache[type_]

        # Determine strategy
        info = self._compute_type_info(type_)
        self._type_cache[type_] = info

        return info

    def get_strategy(self, type_: type) -> TypeStrategy:
        """Get the handling strategy for a type.

        Args:
            type_: The type to get strategy for

        Returns:
            The TypeStrategy enum value
        """
        return self.get_type_info(type_).strategy

    def serialize(self, obj: Any) -> JsonValue:
        """Serialize an object using the appropriate adapter.

        Args:
            obj: The object to serialize

        Returns:
            JSON-compatible value

        Raises:
            TypeError: If no adapter is registered for the type
            ValueError: If the object cannot be serialized
        """
        obj_type = type(obj)
        info = self.get_type_info(obj_type)

        if info.strategy == TypeStrategy.PASSTHROUGH:
            return cast(JsonValue, obj)

        if info.strategy == TypeStrategy.ADAPTER and info.adapter:
            if not info.adapter.can_serialize(obj):
                raise ValueError(f"Adapter cannot serialize this {obj_type.__name__}")
            return info.adapter.serialize(obj)

        if info.strategy == TypeStrategy.OBJECT_STORE:
            raise TypeError(
                f"Type {obj_type.__name__} uses object store strategy. "
                "Use ObjectStore.store() instead of serialize()."
            )

        raise TypeError(f"No serialization strategy for type {obj_type.__name__}")

    def deserialize(self, data: JsonValue, target_type: type[T]) -> T:
        """Deserialize data to the target type.

        Args:
            data: The JSON-compatible data
            target_type: The type to deserialize to

        Returns:
            The deserialized object

        Raises:
            TypeError: If no adapter is registered for the type
        """
        info = self.get_type_info(target_type)

        if info.strategy == TypeStrategy.PASSTHROUGH:
            return data  # type: ignore[return-value]

        if info.strategy == TypeStrategy.ADAPTER and info.adapter:
            return cast(T, info.adapter.deserialize(data))

        if info.strategy == TypeStrategy.OBJECT_STORE:
            raise TypeError(
                f"Type {target_type.__name__} uses object store strategy. "
                "Use ObjectStore.get() instead of deserialize()."
            )

        raise TypeError(f"No deserialization strategy for type {target_type.__name__}")

    def get_json_schema(self, type_: type) -> dict[str, Any]:
        """Get JSON Schema for a type's serialized form.

        Args:
            type_: The type to get schema for

        Returns:
            JSON Schema dictionary
        """
        info = self.get_type_info(type_)
        return info.json_schema.copy()

    def has_adapter(self, type_: type) -> bool:
        """Check if an adapter is registered for a type.

        Args:
            type_: The type to check

        Returns:
            True if an adapter exists
        """
        return self.get_adapter(type_) is not None

    def has_store_config(self, type_: type) -> bool:
        """Check if a store config is registered for a type.

        Args:
            type_: The type to check

        Returns:
            True if a store config exists
        """
        return self.get_store_config(type_) is not None

    def list_adapters(self) -> list[type]:
        """List all types with registered adapters.

        Returns:
            List of type objects
        """
        return list(self._adapters.keys())

    def list_stored_types(self) -> list[type]:
        """List all types configured for object store.

        Returns:
            List of type objects
        """
        return list(self._store_configs.keys())

    def clear(self) -> None:
        """Clear all registered adapters and configs."""
        self._adapters.clear()
        self._store_configs.clear()
        self._type_cache.clear()

    def _compute_type_info(self, type_: type) -> TypeInfo:
        """Compute TypeInfo for a type.

        Args:
            type_: The type to compute info for

        Returns:
            Computed TypeInfo
        """
        # Check if JSON-compatible (passthrough)
        if is_json_compatible(type_):
            return TypeInfo(
                type_=type_,
                strategy=TypeStrategy.PASSTHROUGH,
                json_schema=self._get_primitive_schema(type_),
            )

        # Check for registered adapter
        adapter = self.get_adapter(type_)
        if adapter:
            return TypeInfo(
                type_=type_,
                strategy=TypeStrategy.ADAPTER,
                adapter=adapter,
                json_schema=adapter.json_schema(),
            )

        # Check for object store config
        store_config = self.get_store_config(type_)
        if store_config:
            return TypeInfo(
                type_=type_,
                strategy=TypeStrategy.OBJECT_STORE,
                store_config=store_config,
                json_schema={"type": "string", "description": f"Handle to {type_.__name__}"},
            )

        # No handler registered
        return TypeInfo(
            type_=type_,
            strategy=TypeStrategy.UNSUPPORTED,
            json_schema={},
        )

    def _get_primitive_schema(self, type_: type) -> dict[str, Any]:
        """Get JSON Schema for primitive types.

        Args:
            type_: The primitive type

        Returns:
            JSON Schema dictionary
        """
        if type_ is type(None) or type_ is None:
            return {"type": "null"}
        if type_ is bool:
            return {"type": "boolean"}
        if type_ is int:
            return {"type": "integer"}
        if type_ is float:
            return {"type": "number"}
        if type_ is str:
            return {"type": "string"}
        if type_ is list:
            return {"type": "array"}
        if type_ is dict:
            return {"type": "object"}
        return {}

    def _invalidate_cache(self, type_: type) -> None:
        """Invalidate the cache for a type.

        Args:
            type_: The type to invalidate
        """
        self._type_cache.pop(type_, None)


# Global default registry
_default_registry: TypeRegistry | None = None


def get_default_registry() -> TypeRegistry:
    """Get the default global type registry.

    Returns:
        The default TypeRegistry instance
    """
    global _default_registry
    if _default_registry is None:
        _default_registry = TypeRegistry()
    return _default_registry


def set_default_registry(registry: TypeRegistry) -> None:
    """Set the default global type registry.

    Args:
        registry: The registry to use as default
    """
    global _default_registry
    _default_registry = registry
